#!/usr/bin/env python3


#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import argparse
import csv
import importlib
import sys
from collections import defaultdict
from pathlib import Path

from polib import pofile

WRAP_WIDTH = 99


class LocalizationHelper:
    def __init__(self) -> None:
        arg_parser = argparse.ArgumentParser()
        arg_parser.add_argument("country_code", help="Country code", type=str)
        arg_parser.add_argument("--create-po", action="store_true", help="Create .po files")
        arg_parser.add_argument("--export-csv", action="store_true", help="Export to .csv")
        arg_parser.add_argument("--import-csv", action="store_true", help="Import from .csv")
        arg_parser.add_argument(
            "--fill-missing",
            action="store_true",
            help="Fill missing translations from other countries",
        )
        arg_parser.add_argument(
            "--bom",
            action=argparse.BooleanOptionalAction,
            default=False,
            help="Use files with BOM on export/import",
        )
        arg_parser.add_argument(
            "--overwrite",
            action=argparse.BooleanOptionalAction,
            default=False,
            help="Overwrite existing files",
        )
        arg_parser.add_argument(
            "--strict-locale",
            action=argparse.BooleanOptionalAction,
            default=True,
            help="Use strict locale when searching for suitable translations",
        )
        self.args = arg_parser.parse_args()
        if not any(
            getattr(self.args, action.dest)
            for action in arg_parser._actions
            if isinstance(action, argparse._StoreTrueAction)
        ):
            print("No action specified.\n")
            arg_parser.print_help()
            sys.exit(1)

        self.country_code = self.args.country_code
        self.encoding = "utf-8-sig" if self.args.bom else "utf-8"
        self.csv_file_path = Path(f"holidays/locale/csv/{self.country_code}.csv")

    def create_po_files(self):
        sys.path.append(str(Path.cwd()))  # Make holidays visible.
        from holidays.registry import COUNTRIES

        class_name = None
        for module_name, entity_data in COUNTRIES.items():
            if entity_data[1] == self.country_code:
                class_name = entity_data[0]
                break
        if not class_name:
            print(f"Class for country code {self.country_code} not found!")
            sys.exit(1)

        cls = getattr(importlib.import_module(f"holidays.countries.{module_name}"), class_name)
        docstring = cls.__doc__

        header = [
            f"# {line}" if line else "#"
            for line in Path("docs/file_header.txt").read_text(encoding="utf-8").split("\n")
        ]
        country_name = "COUNTRY"
        if (title := docstring.split("\n")[0]) and title.endswith(" holidays."):
            country_name = title.split("holidays")[0].strip()

        locale_directory = Path("holidays/locale")
        pot_file_path = locale_directory / "pot" / f"{self.country_code}.pot"
        pot_file = pofile(pot_file_path, wrapwidth=WRAP_WIDTH)
        for entry in pot_file:
            entry.occurrences.clear()

        for language in cls.supported_languages:
            po_directory = locale_directory / language / "LC_MESSAGES"
            po_directory.mkdir(parents=True, exist_ok=True)
            po_file_path = po_directory / f"{self.country_code}.po"
            if not self.args.overwrite and po_file_path.exists():
                continue
            pot_file.metadata["Language"] = language
            pot_file.save(po_file_path, newline="\n")

            file_content = po_file_path.read_text(encoding="utf-8").split("\n")
            if file_content[0] != "# SOME DESCRIPTIVE TITLE":
                continue
            l10n_suffix = f" {language} localization" if language != cls.default_language else ""
            title = f"# {country_name} holidays{l10n_suffix}."
            file_content = header + [title, "#"] + file_content[4:]
            po_file_path.write_text("\n".join(file_content), encoding="utf-8", newline="\n")

        if cls.default_language == "en_US":
            return None

        en_us_po_path = locale_directory / "en_us/LC_MESSAGES" / f"{self.country_code}.po"
        po_file = pofile(en_us_po_path, wrapwidth=WRAP_WIDTH)
        for po_entry in po_file:
            if po_entry.comment:
                po_entry.msgstr = po_entry.comment.removesuffix(".")
        po_file.save(en_us_po_path, newline="\n")

    def export_to_csv(self):
        if not self.args.overwrite and self.csv_file_path.exists():
            print(f"File {self.csv_file_path} exists and no --overwrite option!")
            sys.exit(1)

        comments = {}
        languages = []
        po_data = defaultdict(dict)
        for po_path in Path("holidays/locale").rglob(f"{self.country_code}.po"):
            po_file = pofile(po_path, wrapwidth=WRAP_WIDTH)
            language = po_file.metadata["Language"]
            default_language = po_file.metadata["X-Source-Language"]
            if language != default_language:
                languages.append(language)
                for po_entry in po_file:
                    po_data[po_entry.msgid][language] = po_entry.msgstr
            else:
                for po_entry in po_file:
                    comments[po_entry.msgid] = po_entry.comment

        languages = sorted(languages)
        headers = ["msgid", "msgcomment", *languages]
        rows = [
            [msgid, comments[msgid]] + [po_data[msgid][language] for language in languages]
            for msgid in po_data
        ]
        self.write_csv(headers, rows)

    def read_csv(self) -> tuple[list[str], list[list[str]]]:
        if not self.csv_file_path.exists():
            print(f"File {self.csv_file_path} not found!")
            sys.exit(1)

        with self.csv_file_path.open("r", encoding=self.encoding) as f:
            reader = csv.reader(f, delimiter=";")
            headers, *rows = reader
        return headers, rows

    def write_csv(self, headers: list[str], rows: list[list[str]]):
        self.csv_file_path.parent.mkdir(parents=True, exist_ok=True)
        with self.csv_file_path.open("w", newline="", encoding=self.encoding) as f:
            writer = csv.writer(f, delimiter=";")
            writer.writerow(headers)
            writer.writerows(rows)

    def import_from_csv(self):
        headers, rows = self.read_csv()
        languages = headers[2:]
        entries = [(msgid, dict(zip(languages, msgstrs))) for msgid, _, *msgstrs in rows]

        for language in languages:
            po_path = Path(f"holidays/locale/{language}/LC_MESSAGES/{self.country_code}.po")
            po_file = pofile(po_path, wrapwidth=WRAP_WIDTH)

            changed = False
            for msgid, msg_mapping in entries:
                entry = po_file.find(msgid)
                if not entry:
                    continue
                msgstr = msg_mapping[language]
                if entry.msgstr != msgstr:
                    entry.msgstr = msgstr
                    if "|" in msgstr:
                        entry.flags.append("fuzzy")
                    changed = True

            if changed:
                po_file.save(po_path, newline="\n")

    def fill_missing_translations(self):
        headers, rows = self.read_csv()
        languages = headers[2:]

        untranslated_languages = set()
        for row in rows:
            msgstrs = row[2:]
            for lang, msgstr in zip(languages, msgstrs):
                if not msgstr.strip():
                    untranslated_languages.add(lang)

        locale_path = Path("holidays/locale")
        translations = {language: defaultdict(set) for language in untranslated_languages}
        for language in untranslated_languages:
            language_prefix = language if self.args.strict_locale else language.split("_")[0]
            for po_path in locale_path.rglob("LC_MESSAGES/*.po"):
                if po_path.stem == self.country_code:
                    continue
                language_dir = po_path.parent.parent.name
                if not (
                    language_dir == language_prefix
                    if self.args.strict_locale
                    else language_dir.startswith(language_prefix)
                ):
                    continue
                po_file = pofile(po_path, wrapwidth=WRAP_WIDTH)
                for entry in po_file:
                    if entry.comment:
                        translations[language][entry.comment].add(entry.msgstr or entry.msgid)

        updated_rows = []
        for msgid, msgcomment, *msgstrs in rows:
            new_msgstrs = []
            for lang, msgstr in zip(languages, msgstrs):
                if msgstr.strip():
                    new_msgstrs.append(msgstr)
                elif msgcomment in translations[lang]:
                    new_msgstrs.append(" | ".join(sorted(translations[lang][msgcomment])))
                else:
                    new_msgstrs.append("")

            updated_rows.append([msgid, msgcomment, *new_msgstrs])

        self.write_csv(headers, updated_rows)

    def run(self):
        if self.args.create_po:
            self.create_po_files()
        elif self.args.export_csv:
            self.export_to_csv()
        elif self.args.import_csv:
            self.import_from_csv()
        elif self.args.fill_missing:
            self.fill_missing_translations()


if __name__ == "__main__":
    LocalizationHelper().run()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import BANK, PUBLIC
from holidays.countries.panama import Panama
from tests.common import CommonCountryTests


class TestPanama(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Panama, years_non_observed=range(2000, 2025))

    def test_special_holidays(self):
        self.assertHoliday(
            "2014-07-01",
            "2019-07-01",
            "2024-07-01",
        )

    def test_new_years_day(self):
        name = "Año Nuevo"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2006-01-02",
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (puente)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_martyrs_day(self):
        name = "Día de los Mártires"
        self.assertHolidayName(name, (f"{year}-01-09" for year in range(1972, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1972))
        obs_dts = (
            "2005-01-10",
            "2011-01-10",
            "2022-01-10",
        )
        self.assertHolidayName(f"{name} (puente)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_constitution_day(self):
        name = "Día de la Constitución"
        self.assertHolidayName(name, (f"{year}-03-01" for year in range(self.start_year, 1972)))
        self.assertNoHolidayName(name, range(1972, self.end_year))

    def test_carnival_monday(self):
        name = "Lunes de Carnaval"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            "2020-02-24",
            "2021-02-15",
            "2022-02-28",
            "2023-02-20",
            "2024-02-12",
            "2025-03-03",
        )
        self.assertBankHolidayName(name, self.full_range)

    def test_carnival_tuesday(self):
        name = "Martes de Carnaval"
        self.assertHolidayName(
            name,
            "2020-02-25",
            "2021-02-16",
            "2022-03-01",
            "2023-02-21",
            "2024-02-13",
            "2025-03-04",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Día del Trabajo"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2005-05-02",
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (puente)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_separation_day(self):
        name = "Separación de Panamá de Colombia"
        self.assertHolidayName(name, (f"{year}-11-03" for year in self.full_range))
        obs_dts = (
            "2002-11-04",
            "2013-11-04",
            "2019-11-04",
            "2024-11-04",
        )
        self.assertHolidayName(f"{name} (puente)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_colon_day(self):
        name = "Día de Colón"
        self.assertHolidayName(name, (f"{year}-11-05" for year in range(2002, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2002))
        obs_dts = (
            "2006-11-06",
            "2017-11-06",
            "2023-11-06",
        )
        self.assertHolidayName(f"{name} (puente)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_los_santos_uprising_day(self):
        name = "Primer Grito de Independencia"
        self.assertHolidayName(name, (f"{year}-11-10" for year in range(1969, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1969))
        obs_dts = (
            "2002-11-11",
            "2013-11-11",
            "2019-11-11",
            "2024-11-11",
        )
        self.assertHolidayName(f"{name} (puente)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Independencia de Panamá de España"
        self.assertHolidayName(name, (f"{year}-11-28" for year in self.full_range))
        obs_dts = (
            "2004-11-29",
            "2010-11-29",
            "2021-11-29",
        )
        self.assertHolidayName(f"{name} (puente)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_mothers_day(self):
        name = "Día de la Madre"
        self.assertHolidayName(name, (f"{year}-12-08" for year in self.full_range))
        obs_dts = (
            "2002-12-09",
            "2013-12-09",
            "2019-12-09",
            "2024-12-09",
        )
        self.assertHolidayName(f"{name} (puente)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_mourning_day(self):
        name = "Día de Duelo Nacional"
        self.assertHolidayName(name, (f"{year}-12-20" for year in range(2022, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2022))
        obs_dts = ("2026-12-21",)
        self.assertHolidayName(f"{name} (puente)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Navidad"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2005-12-26",
            "2011-12-26",
            "2016-12-26",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (puente)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_symbols_day(self):
        name = "Día de los Símbolos Patrios"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-11-04" for year in self.full_range))

    def test_2022(self):
        self.assertHolidays(
            Panama(categories=(BANK, PUBLIC), years=2022),
            ("2022-01-01", "Año Nuevo"),
            ("2022-01-09", "Día de los Mártires"),
            ("2022-01-10", "Día de los Mártires (puente)"),
            ("2022-02-28", "Lunes de Carnaval"),
            ("2022-03-01", "Martes de Carnaval"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-05-01", "Día del Trabajo"),
            ("2022-05-02", "Día del Trabajo (puente)"),
            ("2022-11-03", "Separación de Panamá de Colombia"),
            ("2022-11-04", "Día de los Símbolos Patrios"),
            ("2022-11-05", "Día de Colón"),
            ("2022-11-10", "Primer Grito de Independencia"),
            ("2022-11-28", "Independencia de Panamá de España"),
            ("2022-12-08", "Día de la Madre"),
            ("2022-12-20", "Día de Duelo Nacional"),
            ("2022-12-25", "Navidad"),
            ("2022-12-26", "Navidad (puente)"),
        )

    def test_2023(self):
        self.assertHolidays(
            Panama(categories=(BANK, PUBLIC), years=2023),
            ("2023-01-01", "Año Nuevo"),
            ("2023-01-02", "Año Nuevo (puente)"),
            ("2023-01-09", "Día de los Mártires"),
            ("2023-02-20", "Lunes de Carnaval"),
            ("2023-02-21", "Martes de Carnaval"),
            ("2023-04-07", "Viernes Santo"),
            ("2023-05-01", "Día del Trabajo"),
            ("2023-11-03", "Separación de Panamá de Colombia"),
            ("2023-11-04", "Día de los Símbolos Patrios"),
            ("2023-11-05", "Día de Colón"),
            ("2023-11-06", "Día de Colón (puente)"),
            ("2023-11-10", "Primer Grito de Independencia"),
            ("2023-11-28", "Independencia de Panamá de España"),
            ("2023-12-08", "Día de la Madre"),
            ("2023-12-20", "Día de Duelo Nacional"),
            ("2023-12-25", "Navidad"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Año Nuevo"),
            ("2024-01-09", "Día de los Mártires"),
            ("2024-02-12", "Lunes de Carnaval"),
            ("2024-02-13", "Martes de Carnaval"),
            ("2024-03-29", "Viernes Santo"),
            ("2024-05-01", "Día del Trabajo"),
            ("2024-07-01", "Toma posesión del Presidente de la república"),
            ("2024-11-03", "Separación de Panamá de Colombia"),
            (
                "2024-11-04",
                "Día de los Símbolos Patrios; Separación de Panamá de Colombia (puente)",
            ),
            ("2024-11-05", "Día de Colón"),
            ("2024-11-10", "Primer Grito de Independencia"),
            ("2024-11-11", "Primer Grito de Independencia (puente)"),
            ("2024-11-28", "Independencia de Panamá de España"),
            ("2024-12-08", "Día de la Madre"),
            ("2024-12-09", "Día de la Madre (puente)"),
            ("2024-12-20", "Día de Duelo Nacional"),
            ("2024-12-25", "Navidad"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-09", "Martyrs' Day"),
            ("2024-02-12", "Carnival Monday"),
            ("2024-02-13", "Carnival Tuesday"),
            ("2024-03-29", "Good Friday"),
            ("2024-05-01", "Labor Day"),
            ("2024-07-01", "Presidential Inauguration Day"),
            ("2024-11-03", "Separation Day"),
            ("2024-11-04", "National Symbols Day; Separation Day (observed)"),
            ("2024-11-05", "Colon Day"),
            ("2024-11-10", "Los Santos Uprising Day"),
            ("2024-11-11", "Los Santos Uprising Day (observed)"),
            ("2024-11-28", "Independence Day"),
            ("2024-12-08", "Mother's Day"),
            ("2024-12-09", "Mother's Day (observed)"),
            ("2024-12-20", "National Mourning Day"),
            ("2024-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2024-01-01", "Новий рік"),
            ("2024-01-09", "День мучеників"),
            ("2024-02-12", "Карнавальний понеділок"),
            ("2024-02-13", "Карнавальний вівторок"),
            ("2024-03-29", "Страсна пʼятниця"),
            ("2024-05-01", "День праці"),
            ("2024-07-01", "Інавгурація Президента Республіки"),
            ("2024-11-03", "День відокремлення від Колумбії"),
            (
                "2024-11-04",
                "День відокремлення від Колумбії (вихідний); День національних символів",
            ),
            ("2024-11-05", "День Колона"),
            ("2024-11-10", "День початку повстання у Лос-Сантос"),
            ("2024-11-11", "День початку повстання у Лос-Сантос (вихідний)"),
            ("2024-11-28", "День незалежності від Іспанії"),
            ("2024-12-08", "День матері"),
            ("2024-12-09", "День матері (вихідний)"),
            ("2024-12-20", "День національної скорботи"),
            ("2024-12-25", "Різдво Христове"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import importlib
import os
import re
import sys
import warnings
from collections import defaultdict
from collections.abc import Generator
from datetime import date
from functools import cache

from dateutil.parser import parse

from holidays import HolidayBase
from holidays.calendars.gregorian import SUN
from holidays.constants import PUBLIC
from holidays.groups import EasternCalendarHolidays, IslamicHolidays
from holidays.observed_holiday_base import ObservedHolidayBase
from holidays.registry import COUNTRIES, FINANCIAL

PYTHON_LATEST_SUPPORTED_VERSION = "3.14"
PYTHON_VERSION = f"{sys.version_info.major}.{sys.version_info.minor}"


class TestCase:
    """Base class for holidays test cases."""

    @staticmethod
    @cache
    def _get_or_create_lookup(
        test_class, *, with_subdiv_categories=False, with_subdiv_special_flags=False
    ):
        """Build and cache categories and subdivision lookup tables for a holiday test class."""

        non_public_supported_categories = tuple(
            category for category in test_class.supported_categories if category != PUBLIC
        )

        supported_special_flags_list = []
        if issubclass(test_class, IslamicHolidays):
            supported_special_flags_list.append("islamic_no_estimated")
        supported_special_flags = tuple(supported_special_flags_list)

        subdiv_base = {}
        subdiv_category = {}
        subdiv_special_flag = {}
        for subdiv in test_class.subdivisions:
            subdiv_code = subdiv.lower().replace(" ", "_")
            subdiv_base[subdiv_code] = (subdiv, None, subdiv_code)

            if with_subdiv_categories:
                for category in non_public_supported_categories:
                    subdiv_category[f"{subdiv_code}_{category}"] = (subdiv, category, subdiv_code)

            if with_subdiv_special_flags:
                for special_flag in supported_special_flags:
                    subdiv_special_flag[f"{subdiv_code}_{special_flag}"] = (
                        subdiv,
                        special_flag,
                        subdiv_code,
                    )

        return (
            subdiv_base,
            subdiv_category,
            non_public_supported_categories,
            subdiv_special_flag,
            supported_special_flags,
        )

    @classmethod
    def _generate_assert_methods(cls):
        """Dynamically generate assertion methods for all holiday variants."""

        def make_assert(helper_func, instance_name, method_type):
            def _method(self, *args, **kwargs):
                match method_type:
                    case "name_count":
                        if len(args) < 2:
                            raise TypeError(
                                f"Expected (name, count, ...) arguments for {instance_name}"
                            )
                        name, count, *rest = args
                        return helper_func(self, name, count, instance_name, *rest, **kwargs)
                    case "name":
                        if not args:
                            raise TypeError(f"Expected (name, ...) arguments for {instance_name}")
                        name, *rest = args
                        return helper_func(self, name, instance_name, *rest, **kwargs)
                    case "year":
                        if not args:
                            raise TypeError(f"Expected (year, ...) arguments for {instance_name}")
                        year, *rest = args
                        return helper_func(self, year, instance_name, *rest, **kwargs)
                    case _:
                        return helper_func(self, instance_name, *args, **kwargs)

            return _method

        method_specs = {
            "_assertHoliday": "assert{variant}Holiday",
            "_assertHolidayDates": "assert{variant}HolidayDates",
            "_assertHolidayDatesInYear": "assert{variant}HolidayDatesInYear",
            "_assertHolidayName": "assert{variant}HolidayName",
            "_assertHolidayNameCount": "assert{variant}HolidayNameCount",
            "_assertHolidays": "assert{variant}Holidays",
            "_assertHolidaysInYear": "assert{variant}HolidaysInYear",
            "_assertNoHoliday": "assertNo{variant}Holiday",
            "_assertNoHolidayName": "assertNo{variant}HolidayName",
            "_assertNoHolidays": "assertNo{variant}Holidays",
        }

        method_types = {
            "_assertHolidayDatesInYear": "year",
            "_assertHolidayNameCount": "name_count",
            "_assertHolidayName": "name",
            "_assertHolidaysInYear": "year",
            "_assertNoHolidayName": "name",
        }

        for attr_name in dir(cls):
            if not attr_name.startswith("holidays"):
                continue

            variant = attr_name.removeprefix("holidays")
            pretty_variant = "".join(part.capitalize() for part in variant.strip("_").split("_"))

            for helper_name, template in method_specs.items():
                method_name = template.format(variant=pretty_variant)
                if hasattr(cls, method_name):
                    continue

                helper = getattr(cls, helper_name)
                method_type = method_types.get(helper_name, "default")
                setattr(cls, method_name, make_assert(helper, attr_name, method_type))

    @classmethod
    def setUpClass(
        cls,
        test_class=None,
        *,
        with_subdiv_categories=False,
        with_subdiv_special_flags=False,
        years=None,
        **year_variants,
    ):
        super().setUpClass()

        if test_class is None:
            return None

        cls.test_class = test_class

        default_lang = getattr(cls.test_class, "default_language", None)
        if default_lang is not None:
            # Normally 2-6 letters (e.g., en, pap, en_US, pap_AW).
            if not (2 <= len(default_lang) <= 6):
                raise ValueError(f"`{cls.test_class.__name__}.default_language` value is invalid.")
            cls.set_language(cls.test_class, default_lang)

        # Use cached lookup instead of rebuilding each time.
        (
            cls._subdiv_base_lookup,
            cls._subdiv_category_lookup,
            cls._non_public_supported_categories_lookup,
            cls._subdiv_special_flag_lookup,
            cls._supported_special_flags_lookup,
        ) = cls._get_or_create_lookup(
            cls.test_class,
            with_subdiv_categories=with_subdiv_categories,
            with_subdiv_special_flags=with_subdiv_special_flags,
        )
        cls._subdiv_lookup = cls._subdiv_base_lookup | cls._subdiv_category_lookup

        if years is None:
            # Default `self.full_range`
            if not hasattr(cls, "full_range"):
                # `start_year` and `end_year` are used only if they're included directly in
                # country/market entities, not inherited from `HolidayBase`.
                cls.start_year = cls.test_class.__dict__.get("start_year", 1950)
                cls.end_year = cls.test_class.__dict__.get("end_year", 2049) + 1
                cls.full_range = range(cls.start_year, cls.end_year)
            else:
                cls.start_year = cls.full_range.start
                cls.end_year = cls.full_range.stop
            years = cls.full_range

        # Default `years_[insert]` to `years` to prevent redundant initialization.
        is_observed_subclass = issubclass(cls.test_class, ObservedHolidayBase)
        is_islamic_subclass = issubclass(cls.test_class, IslamicHolidays)
        for category in cls._non_public_supported_categories_lookup:
            year_variants.setdefault(f"years_{category}", years)
            if is_observed_subclass:
                year_variants.setdefault(
                    f"years_{category}_non_observed",
                    year_variants.get(
                        "years_non_observed", year_variants.get(f"years_{category}", years)
                    ),
                )
            if is_islamic_subclass:
                year_variants.setdefault(
                    f"years_{category}_islamic_no_estimated",
                    year_variants.get(
                        "years_islamic_no_estimated", year_variants.get(f"years_{category}", years)
                    ),
                )

        # For subdivisions, `years_all_subdivs` can be use for mass-assignments.
        years_all_subdivs = year_variants.get("years_all_subdivs", years)
        years_all_subdivs_non_observed = year_variants.get(
            "years_all_subdivs_non_observed",
            year_variants.get("years_non_observed", year_variants.get("years_all_subdivs", years)),
        )

        for key, (_, _, subdiv_code) in (
            *cls._subdiv_lookup.items(),
            *cls._subdiv_special_flag_lookup.items(),
        ):
            year_variants.setdefault(
                f"years_subdiv_{key}",
                year_variants.get(f"years_subdiv_{subdiv_code}", years_all_subdivs),
            )
            if is_observed_subclass:
                year_variants.setdefault(
                    f"years_subdiv_{key}_non_observed",
                    year_variants.get(
                        f"years_subdiv_{subdiv_code}_non_observed", years_all_subdivs_non_observed
                    ),
                )

        if is_islamic_subclass:
            year_variants.setdefault("years_islamic_no_estimated", years)
        if is_observed_subclass:
            year_variants.setdefault("years_non_observed", years)
        if is_islamic_subclass and is_observed_subclass:
            year_variants.setdefault("years_islamic_no_estimated_non_observed", years)

        variants = {"": years}
        variants.update(year_variants)

        for suffix, ylist in variants.items():
            # Exclude invalid cases & mass-assignment helpers.
            if ylist is None or suffix.startswith("years_all_subdivs"):
                continue

            attr_name_suffix = ""
            init_kwargs = {"years": ylist}

            # Step 1: `_non_observed` suffix.
            if suffix.endswith("_non_observed"):
                suffix = suffix.removesuffix("_non_observed")
                attr_name_suffix = "_non_observed"
                init_kwargs["observed"] = False

            # Step 2: Special flags i.e. `islamic_show_estimated`.
            if suffix.endswith("_islamic_no_estimated"):
                suffix = suffix.removesuffix("_islamic_no_estimated")
                init_kwargs["islamic_show_estimated"] = False
                attr_name_suffix = f"_islamic_no_estimated{attr_name_suffix}"

            # Step 3: Subdivisions
            if suffix.startswith("years_subdiv_"):
                key = suffix.removeprefix("years_subdiv_")
                if key in cls._subdiv_lookup:
                    subdiv, category, subdiv_code = cls._subdiv_lookup[key]
                    init_kwargs["subdiv"] = subdiv
                    if category:
                        init_kwargs["categories"] = (category,)
                        attr_name_suffix = f"_{category}{attr_name_suffix}"
                    attr_name_suffix = f"_subdiv_{subdiv_code}{attr_name_suffix}"

            # Step 4: Categories
            elif suffix.startswith("years_"):
                category = suffix.removeprefix("years_")
                init_kwargs["categories"] = (category,)
                attr_name_suffix = f"_{category}{attr_name_suffix}"

            attr_name = f"holidays{attr_name_suffix}"
            setattr(cls, attr_name, cls.test_class(**init_kwargs))

        # Legacy `cls.subdiv_holidays` / `cls.subdiv_holidays_non_observed` behavior.
        cls.subdiv_holidays = {}
        cls.subdiv_holidays_non_observed = {}
        for subdiv, _, subdiv_code in cls._subdiv_base_lookup.values():
            key_subdiv = f"holidays_subdiv_{subdiv_code}"
            key_subdiv_non_obs = f"{key_subdiv}_non_observed"

            if hasattr(cls, key_subdiv):
                cls.subdiv_holidays[subdiv] = getattr(cls, key_subdiv)
            if is_observed_subclass and hasattr(cls, key_subdiv_non_obs):
                cls.subdiv_holidays_non_observed[subdiv] = getattr(cls, key_subdiv_non_obs)

        if with_subdiv_categories:
            dict_subdiv_cat = defaultdict(dict)
            dict_subdiv_cat_non_obs = defaultdict(dict)
            for category in cls._non_public_supported_categories_lookup:
                setattr(cls, f"subdiv_{category}_holidays", dict_subdiv_cat[category])

                if is_observed_subclass:
                    setattr(
                        cls,
                        f"subdiv_{category}_holidays_non_observed",
                        dict_subdiv_cat_non_obs[category],
                    )

            for subdiv, category, subdiv_code in cls._subdiv_category_lookup.values():
                key_subdiv_cat = f"holidays_subdiv_{subdiv_code}_{category}"
                dict_subdiv_cat[category][subdiv] = getattr(cls, key_subdiv_cat, {})

                if is_observed_subclass:
                    key_subdiv_cat_non_obs = f"{key_subdiv_cat}_non_observed"
                    dict_subdiv_cat_non_obs[category][subdiv] = getattr(
                        cls, key_subdiv_cat_non_obs, {}
                    )

        if with_subdiv_special_flags:
            dict_subdiv_special_flag = defaultdict(dict)
            dict_subdiv_special_flag_non_obs = defaultdict(dict)
            for special_flag in cls._supported_special_flags_lookup:
                setattr(
                    cls, f"subdiv_{special_flag}_holidays", dict_subdiv_special_flag[special_flag]
                )

                if is_observed_subclass:
                    setattr(
                        cls,
                        f"subdiv_{special_flag}_holidays_non_observed",
                        dict_subdiv_special_flag_non_obs[special_flag],
                    )

            for subdiv, special_flag, subdiv_code in cls._subdiv_special_flag_lookup.values():
                key_subdiv_special_flag = f"holidays_subdiv_{subdiv_code}_{special_flag}"
                dict_subdiv_special_flag[special_flag][subdiv] = getattr(
                    cls, key_subdiv_special_flag, {}
                )

                if is_observed_subclass:
                    key_subdiv_special_flag_non_obs = f"{key_subdiv_special_flag}_non_observed"
                    dict_subdiv_special_flag_non_obs[special_flag][subdiv] = getattr(
                        cls, key_subdiv_special_flag_non_obs, {}
                    )

        cls._generate_assert_methods()

    def setUp(self):
        super().setUp()

        if getattr(self.test_class, "default_language") is not None:
            self.set_language(self.test_class.default_language)

    def set_language(self, language):
        os.environ["LANGUAGE"] = language

    def _parse_arguments(
        self, args, *, expand_items=True, instance_name="holidays", raise_on_empty=True
    ):
        item_args = args
        instance = None

        if args and isinstance(args[0], HolidayBase):
            instance = args[0]
            item_args = args[1:]
        else:
            try:
                instance = getattr(self, instance_name)
                self.assertIsInstance(
                    instance,
                    HolidayBase,
                    f"The `self.{instance_name}` must be a `HolidayBase` subclass.",
                )
            except AttributeError:
                raise ValueError(
                    "Either pass a holidays object (`HolidayBase` subclass) "
                    "as a first argument or initialize your `TestCase` class "
                    "properly with `setUpClass()` method."
                )

        items = []
        if expand_items:
            for item_arg in item_args:
                match item_arg:
                    case list() | set() | tuple():
                        items.extend(item_arg)
                    case Generator() | range():
                        items.extend(tuple(item_arg))
                    case _:
                        items.append(item_arg)
        else:
            items.extend(item_args)

        if instance_name.endswith("_non_observed"):
            self.assertFalse(instance.observed)
        else:
            self.assertTrue(instance.observed)

        if raise_on_empty and not items:
            raise ValueError("The test argument sequence is empty")

        return instance, items

    def _verify_type(self, holidays):
        self.assertTrue(
            issubclass(holidays.__class__, HolidayBase),
            "`holidays` object must be a subclass of `HolidayBase`",
        )

    def assertAliases(self, *aliases):
        """Assert aliases match the test class."""
        self.assertTrue(aliases, "Entity class does not have any aliases.")
        self.assertTrue(
            issubclass(self.test_class, HolidayBase),
            "The entity object must be a subclass of `HolidayBase`",
        )

        type_error_message = "The entity alias object must be a subclass of the entity class."
        for alias in aliases:
            self.assertIsNotNone(alias, type_error_message)
            self.assertTrue(issubclass(alias, self.test_class), type_error_message)

    def assertDeprecatedSubdivisions(self, message):
        warnings.simplefilter("always", category=DeprecationWarning)
        for subdiv in self.test_class._deprecated_subdivisions:
            with warnings.catch_warnings(record=True) as ctx:
                self.test_class(subdiv=subdiv)
                warning = ctx[0]
                self.assertTrue(issubclass(warning.category, DeprecationWarning))
                self.assertIn(message, str(warning.message))

    # Holiday.
    def _assertHoliday(self, instance_name, *args):
        """Helper: assert each date is a holiday."""
        holidays, dates = self._parse_arguments(args, instance_name=instance_name)
        self._verify_type(holidays)

        for dt in dates:
            self.assertIn(dt, holidays, dt)

    # HolidayDates.
    def _assertHolidayDates(self, instance_name, *args):
        """Helper: assert holiday dates exactly match expected dates."""
        holidays, dates = self._parse_arguments(args, instance_name=instance_name)
        self._verify_type(holidays)

        # Check one by one for descriptive error messages.
        for dt in dates:
            self.assertIn(dt, holidays, dt)

        self.assertEqual(len(dates), len(holidays.keys()), set(dates).difference(holidays.keys()))

    # HolidayDatesInYear.
    def _assertHolidayDatesInYear(self, year, instance_name, *args):
        """Helper: assert holiday dates exactly match expected dates in the given year."""
        holidays, dates = self._parse_arguments(args, instance_name=instance_name)
        self._verify_type(holidays)

        filtered_holidays = {dt.strftime("%Y-%m-%d") for dt in holidays.keys() if dt.year == year}

        # Check one by one for descriptive error messages.
        for dt in dates:
            self.assertIn(dt, filtered_holidays, dt)

        self.assertEqual(
            len(dates), len(filtered_holidays), set(dates).difference(filtered_holidays)
        )

    # HolidayName.
    def _assertHolidayName(self, name, instance_name, *args):
        """Helper: assert either a holiday with a specific name exists or
        each holiday name matches an expected one depending on the args nature.
        """
        holidays, items = self._parse_arguments(args, instance_name=instance_name)

        arg = items[0]
        if isinstance(arg, int):  # A holiday name check for a specific year.
            holiday_years = {dt.year for dt in holidays.get_named(name, lookup="exact")}
            self.assertTrue(set(items).issubset(holiday_years), name)
        elif isinstance(arg, date) or parse(arg):  # Exact date check.
            for dt in items:
                self.assertIn(name, holidays.get_list(dt), dt)
        else:
            raise ValueError(f"The {arg} wasn't caught by `assertHolidayName()`")

    # HolidayNameCount.
    def _assertHolidayNameCount(self, name, count, instance_name, *args):
        """Helper: assert number of holidays with a specific name in every year matches
        expected.
        """
        holidays, items = self._parse_arguments(args, instance_name=instance_name)
        self._verify_type(holidays)

        holiday_counts = defaultdict(int)
        for dt in holidays.get_named(name, lookup="exact"):
            holiday_counts[dt.year] += 1

        for year in items:
            self.assertIsInstance(year, int, "Year arguments must be integers")
            holiday_count = holiday_counts.get(year, 0)
            self.assertEqual(
                count,
                holiday_count,
                f"`{name}` occurs {holiday_count} times in year {year}, should be {count}",
            )

    # Holidays.
    def _assertHolidays(self, instance_name, *args):
        """Helper: assert holidays exactly match expected holidays."""
        holidays, expected_holidays = self._parse_arguments(
            args, expand_items=False, instance_name=instance_name
        )
        self._verify_type(holidays)

        # Check one by one for descriptive error messages.
        for dt, name in expected_holidays:
            self.assertIn(dt, holidays)
            self.assertEqual(name, holidays.get(dt), dt)

        self.assertEqual(
            len(holidays),
            len(expected_holidays),
            {(dt.strftime("%Y-%m-%d"), name) for dt, name in holidays.items()}.difference(
                (dt, name) for dt, name in expected_holidays
            ),
        )

    # HolidaysInYear.
    def _assertHolidaysInYear(self, year, instance_name, *args):
        """Helper: assert holidays exactly match expected holidays in the given year."""
        holidays, expected_holidays = self._parse_arguments(
            args, expand_items=False, instance_name=instance_name
        )
        self._verify_type(holidays)

        filtered_holidays = {
            dt.strftime("%Y-%m-%d"): name for dt, name in holidays.items() if dt.year == year
        }

        # Check one by one for descriptive error messages.
        for dt, name in expected_holidays:
            self.assertIn(dt, filtered_holidays)
            self.assertEqual(name, filtered_holidays.get(dt), dt)

        self.assertEqual(
            len(filtered_holidays),
            len(expected_holidays),
            {(dt, name) for dt, name in filtered_holidays.items()}.difference(
                (dt, name) for dt, name in expected_holidays
            ),
        )

    # No Holiday.
    def _assertNoHoliday(self, instance_name, *args):
        """Helper: assert each date is not a holiday."""
        holidays, dates = self._parse_arguments(args, instance_name=instance_name)

        for dt in dates:
            self.assertNotIn(dt, holidays, dt)

    # No HolidayName.
    def _assertNoHolidayName(self, name, instance_name, *args):
        """Helper: assert a holiday with a specific name doesn't exist."""
        holidays, items = self._parse_arguments(
            args, instance_name=instance_name, raise_on_empty=False
        )

        if not items:  # A holiday name check.
            self.assertFalse(holidays.get_named(name, lookup="exact"), name)
            return None

        arg = items[0]
        if isinstance(arg, int):  # A holiday name check for a specific year.
            holiday_years = {dt.year for dt in holidays.get_named(name, lookup="exact")}
            self.assertEqual(0, len(holiday_years.intersection(items)), name)
        elif isinstance(arg, date) or parse(arg):  # Exact date check.
            for dt in items:
                self.assertNotIn(name, holidays.get_list(dt), dt)
        else:
            raise ValueError(f"The {arg} wasn't caught by `assertNoHolidayName()`")

    # No Holidays.
    def _assertNoHolidays(self, instance_name, *args):
        """Helper: assert holidays dict is empty."""
        holidays, _ = self._parse_arguments(
            args, instance_name=instance_name, raise_on_empty=False
        )
        self._verify_type(holidays)

        self.assertFalse(holidays)
        self.assertEqual(0, len(holidays))

    # LocalizedHolidays.
    def _assertLocalizedHolidays(self, localized_holidays, *, categories=None, language=None):
        """Helper: assert localized holidays match expected names."""
        if categories is None:
            categories = self.test_class.supported_categories

        instance = self.test_class(
            years=int(localized_holidays[0][0].split("-")[0]),
            language=language,
            categories=categories,
        )

        for subdiv in instance.subdivisions:
            instance.update(
                self.test_class(
                    subdiv=subdiv,
                    years=instance.years,
                    language=language,
                    categories=categories,
                )
            )

        actual_holidays = tuple(
            sorted((dt.strftime("%Y-%m-%d"), name) for dt, name in instance.items())
        )
        self.assertEqual(
            actual_holidays,
            localized_holidays,
            f"Please make sure all holiday names are localized: {actual_holidays}",
        )

    def assertLocalizedHolidays(self, *args, categories=None):
        """Assert localized holidays match expected names."""
        arg = args[0]
        is_string = isinstance(arg, str)

        language = arg if is_string else None
        localized_holidays = args[1:] if is_string else args

        if language:
            self.set_language(language)
        for language in (language, "invalid", ""):
            self._assertLocalizedHolidays(
                localized_holidays, categories=categories, language=language
            )


class CommonTests(TestCase):
    """Common test cases for all entities."""

    def check_aliases(self, registry, prefix):
        class_name = self.test_class.__name__
        module_name = re.sub(r"(?<!^)(?=[A-Z])", "_", class_name).lower()

        if (entities := registry.get(module_name)) and class_name in entities:
            self.assertAliases(
                *(
                    getattr(
                        importlib.import_module(f"holidays.{prefix}.{module_name}"), alias_name
                    )
                    for alias_name in entities[1:]
                )
            )

    def test_estimated_label(self):
        if isinstance(self.holidays, EasternCalendarHolidays):
            self.assertTrue(
                getattr(self.holidays, "estimated_label", None),
                "The `estimated_label` attribute is required for entities inherited from "
                "`EasternCalendarHolidays`.",
            )

    def test_no_holidays(self):
        self.assertNoHolidays(
            self.test_class(
                categories=self.test_class.supported_categories,
                years=self.test_class.start_year - 1,
            )
        )

    def test_observed_estimated_label(self):
        estimated_label = getattr(self.holidays, "estimated_label", None)
        observed_label = getattr(self.holidays, "observed_label", None)
        observed_estimated_label = getattr(self.holidays, "observed_estimated_label", None)

        if (
            estimated_label
            and observed_label
            # In certain entities, the observed rule applies only to holiday suppression,
            # e.g., XNSE with `SAT_TO_NONE`` or `SUN_TO_NONE``. In these cases, the
            # `observed_estimated_label` is not required.
            and any(
                rule is not None for rule in getattr(self.holidays, "observed_rule", {}).values()
            )
        ):
            self.assertTrue(
                observed_estimated_label,
                "The `observed_estimated_label` attribute is required for entities containing "
                "both `observed_label` and `estimated_label`.",
            )
            self.assertIn(estimated_label.strip("%s ()"), observed_estimated_label)

    def test_observed_label(self):
        if getattr(self.holidays, "observed_label", None):
            self.assertTrue(
                isinstance(self.holidays, ObservedHolidayBase),
                "The `observed_label` attribute is not required as this entity doesn't handle "
                "observed holidays.",
            )

    def test_subdivisions_aliases(self):
        """Validate entity subdivisions aliases."""
        if self.holidays.subdivisions_aliases:
            subdivisions = set(self.holidays.subdivisions)
            for alias, subdiv in self.holidays.subdivisions_aliases.items():
                self.assertIn(
                    subdiv,
                    subdivisions,
                    f"Invalid subdivision alias {alias}: subdivision {subdiv} does not exist.",
                )


class CommonCountryTests(CommonTests):
    """Common test cases for country entities."""

    def test_code(self):
        self.assertTrue(hasattr(self.holidays, "country"))
        self.assertFalse(hasattr(self.holidays, "market"))

    def test_aliases(self):
        """Validate entity aliases."""
        self.check_aliases(COUNTRIES, "countries")


class CommonFinancialTests(CommonTests):
    """Common test cases for financial entities."""

    def test_code(self):
        self.assertTrue(hasattr(self.holidays, "market"))
        self.assertFalse(hasattr(self.holidays, "country"))

    def test_aliases(self):
        """Validate entity aliases."""
        self.check_aliases(FINANCIAL, "financial")


class SundayHolidays(TestCase):
    """Common class to test countries with Sundays as a holidays."""

    def assertSundays(self, cls):
        holidays = cls(years=1989, include_sundays=True)
        self.assertHoliday(holidays, "1989-12-31")
        self.assertEqual(53, len([s for s in holidays if s.weekday() == SUN]))

        holidays = cls(years=2032, include_sundays=True)
        self.assertHoliday(holidays, "2032-01-04")
        self.assertEqual(52, len([s for s in holidays if s.weekday() == SUN]))

        self.assertNoHolidays(cls(include_sundays=True))

        for sunday in (
            "1989-12-31",
            "2017-02-05",
            "2017-02-12",
            "2032-02-29",
        ):
            self.assertEqual(parse(sunday).weekday(), SUN)
            self.assertHoliday(holidays, sunday)

        for non_sunday in (
            "2001-05-16",
            "2001-05-18",
            "2016-12-27",
            "2016-12-28",
            "2017-02-06",
            "2017-02-07",
            "2017-02-08",
            "2017-02-09",
            "2017-02-10",
        ):
            self.assertNotEqual(parse(non_sunday).weekday(), SUN)
            self.assertNoHoliday(holidays, non_sunday)


class WorkingDayTests(TestCase):
    """Common class for testing entity holidays substituted from non-working days."""

    # Workday.
    def _assertWorkingDay(self, instance_name, *args):
        """Helper: assert each date is a working day."""
        holidays, dates = self._parse_arguments(args, instance_name=instance_name)
        self._verify_type(holidays)

        for dt in dates:
            self.assertTrue(
                holidays.is_working_day(dt), f"{dt} should be working day after substitution"
            )

    def assertWorkingDay(self, *args):
        """Assert each date is a working day."""
        self._assertWorkingDay("holidays", *args)

    def assertNonObservedWorkingDay(self, *args):
        """Assert each date is a non-observed working day."""
        self._assertWorkingDay("holidays_non_observed", *args)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.tokelau import Tokelau
from tests.common import CommonCountryTests


class TestTokelau(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Tokelau, years=range(2003, 2050))

    def test_new_years(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in range(2003, 2050)))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(2003, 2050))

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(2003, 2050))

    def test_tokehega_day(self):
        self.assertHolidayName("Tokehega Day", (f"{year}-09-03" for year in range(2003, 2050)))

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in range(2003, 2050)))

    def test_boxing_day(self):
        self.assertHolidayName("Boxing Day", (f"{year}-12-26" for year in range(2003, 2050)))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-09-03", "Tokehega Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-09-03", "Tokehega Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
        )

    def test_l10n_tkl(self):
        self.assertLocalizedHolidays(
            "tkl",
            ("2022-01-01", "Aho Tauhaga Fou"),
            ("2022-04-15", "Ahofalaile Lelei"),
            ("2022-04-18", "Ahogafua o te Eheta"),
            ("2022-09-03", "Aho o te Tokehega"),
            ("2022-12-25", "Aho Kilihimahi"),
            ("2022-12-26", "Tua-aho Kilihimahi"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.iceland import Iceland
from tests.common import CommonCountryTests


class TestIceland(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Iceland)

    def test_new_years_day(self):
        self.assertHolidayName("Nýársdagur", (f"{year}-01-01" for year in self.full_range))

    def test_maundy_thursday(self):
        name = "Skírdagur"
        self.assertHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Föstudagurinn langi"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Páskadagur"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Annar í páskum"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_first_day_of_summer(self):
        name = "Sumardagurinn fyrsti"
        self.assertHolidayName(
            name,
            "2020-04-23",
            "2021-04-22",
            "2022-04-21",
            "2023-04-20",
            "2024-04-25",
            "2025-04-24",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Verkalýðsdagurinn", (f"{year}-05-01" for year in self.full_range))

    def test_ascension_day(self):
        name = "Uppstigningardagur"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_sunday(self):
        name = "Hvítasunnudagur"
        self.assertHolidayName(
            name,
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
            "2024-05-19",
            "2025-06-08",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Annar í hvítasunnu"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_national_day(self):
        self.assertHolidayName(
            "Þjóðhátíðardagurinn", (f"{year}-06-17" for year in self.full_range)
        )

    def test_commerce_day(self):
        name = "Frídagur verslunarmanna"
        self.assertHolidayName(
            name,
            "2020-08-03",
            "2021-08-02",
            "2022-08-01",
            "2023-08-07",
            "2024-08-05",
            "2025-08-04",
        )
        self.assertHolidayName(name, range(1983, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1983))

    def test_christmas_eve(self):
        name = "Aðfangadagur (frá kl. 13.00)"
        self.assertNoHolidayName(name)
        self.assertHalfDayHolidayName(name, (f"{year}-12-24" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Jóladagur", (f"{year}-12-25" for year in self.full_range))

    def test_second_day_of_christmas(self):
        self.assertHolidayName("Annar í jólum", (f"{year}-12-26" for year in self.full_range))

    def test_new_years_eve(self):
        name = "Gamlársdagur (frá kl. 13.00)"
        self.assertNoHolidayName(name)
        self.assertHalfDayHolidayName(name, (f"{year}-12-31" for year in self.full_range))

    def test_2018_public(self):
        self.assertHolidaysInYear(
            2018,
            ("2018-01-01", "Nýársdagur"),
            ("2018-03-29", "Skírdagur"),
            ("2018-03-30", "Föstudagurinn langi"),
            ("2018-04-01", "Páskadagur"),
            ("2018-04-02", "Annar í páskum"),
            ("2018-04-19", "Sumardagurinn fyrsti"),
            ("2018-05-01", "Verkalýðsdagurinn"),
            ("2018-05-10", "Uppstigningardagur"),
            ("2018-05-20", "Hvítasunnudagur"),
            ("2018-05-21", "Annar í hvítasunnu"),
            ("2018-06-17", "Þjóðhátíðardagurinn"),
            ("2018-08-06", "Frídagur verslunarmanna"),
            ("2018-12-25", "Jóladagur"),
            ("2018-12-26", "Annar í jólum"),
        )

    def test_2018_half_day(self):
        self.assertHalfDayHolidaysInYear(
            2018,
            ("2018-12-24", "Aðfangadagur (frá kl. 13.00)"),
            ("2018-12-31", "Gamlársdagur (frá kl. 13.00)"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Nýársdagur"),
            ("2022-04-14", "Skírdagur"),
            ("2022-04-15", "Föstudagurinn langi"),
            ("2022-04-17", "Páskadagur"),
            ("2022-04-18", "Annar í páskum"),
            ("2022-04-21", "Sumardagurinn fyrsti"),
            ("2022-05-01", "Verkalýðsdagurinn"),
            ("2022-05-26", "Uppstigningardagur"),
            ("2022-06-05", "Hvítasunnudagur"),
            ("2022-06-06", "Annar í hvítasunnu"),
            ("2022-06-17", "Þjóðhátíðardagurinn"),
            ("2022-08-01", "Frídagur verslunarmanna"),
            ("2022-12-24", "Aðfangadagur (frá kl. 13.00)"),
            ("2022-12-25", "Jóladagur"),
            ("2022-12-26", "Annar í jólum"),
            ("2022-12-31", "Gamlársdagur (frá kl. 13.00)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-14", "Maundy Thursday"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-21", "First Day of Summer"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-06", "Whit Monday"),
            ("2022-06-17", "National Day"),
            ("2022-08-01", "Commerce Day"),
            ("2022-12-24", "Christmas Eve (from 1pm)"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
            ("2022-12-31", "New Year's Eve (from 1pm)"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-04-14", "Великий четвер"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-04-21", "Перший день літа"),
            ("2022-05-01", "День праці"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-06-05", "Трійця"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-06-17", "Національне свято"),
            ("2022-08-01", "День торгівлі"),
            ("2022-12-24", "Святий вечір (з 13:00)"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
            ("2022-12-31", "Переддень Нового року (з 13:00)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.congo import Congo
from tests.common import CommonCountryTests


class TestCongo(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Congo)

    def test_new_years_day(self):
        self.assertHolidayName("Jour de l'An", (f"{year}-01-01" for year in self.full_range))

    def test_easter_monday(self):
        name = "Lundi de Pâques"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Fête du Travail", (f"{year}-05-01" for year in self.full_range))

    def test_ascension_day(self):
        name = "Ascension"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Lundi de Pentecôte"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_reconciliation_day(self):
        self.assertHolidayName(
            "Fête de la Réconciliation", (f"{year}-06-10" for year in self.full_range)
        )

    def test_national_day(self):
        self.assertHolidayName("Fête Nationale", (f"{year}-08-15" for year in self.full_range))

    def test_all_saints_day(self):
        self.assertHolidayName("Toussaint", (f"{year}-11-01" for year in self.full_range))

    def test_republic_day(self):
        name = "Jour de la République"
        self.assertHolidayName(name, (f"{year}-11-28" for year in range(2010, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2010))

    def test_christmas_day(self):
        self.assertHolidayName("Noël", (f"{year}-12-25" for year in self.full_range))

    def test_2006(self):
        # http://mokili.free.fr/jours_feries.php
        self.assertHolidaysInYear(
            2006,
            ("2006-01-01", "Jour de l'An"),
            ("2006-04-17", "Lundi de Pâques"),
            ("2006-05-01", "Fête du Travail"),
            ("2006-05-25", "Ascension"),
            ("2006-06-05", "Lundi de Pentecôte"),
            ("2006-06-10", "Fête de la Réconciliation"),
            ("2006-08-15", "Fête Nationale"),
            ("2006-11-01", "Toussaint"),
            ("2006-12-25", "Noël"),
        )

    def test_2010(self):
        # http://mokili.free.fr/jours_feries.php
        self.assertHolidaysInYear(
            2010,
            ("2010-01-01", "Jour de l'An"),
            ("2010-04-05", "Lundi de Pâques"),
            ("2010-05-01", "Fête du Travail"),
            ("2010-05-13", "Ascension"),
            ("2010-05-24", "Lundi de Pentecôte"),
            ("2010-06-10", "Fête de la Réconciliation"),
            ("2010-08-15", "Fête Nationale"),
            ("2010-11-01", "Toussaint"),
            ("2010-11-28", "Jour de la République"),
            ("2010-12-25", "Noël"),
        )

    def test_2015(self):
        # http://mokili.free.fr/jours_feries.php
        self.assertHolidaysInYear(
            2015,
            ("2015-01-01", "Jour de l'An"),
            ("2015-04-06", "Lundi de Pâques"),
            ("2015-05-01", "Fête du Travail"),
            ("2015-05-14", "Ascension"),
            ("2015-05-25", "Lundi de Pentecôte"),
            ("2015-06-10", "Fête de la Réconciliation"),
            ("2015-08-15", "Fête Nationale"),
            ("2015-11-01", "Toussaint"),
            ("2015-11-28", "Jour de la République"),
            ("2015-12-25", "Noël"),
        )

    def test_2016(self):
        # http://mokili.free.fr/jours_feries.php
        self.assertHolidaysInYear(
            2016,
            ("2016-01-01", "Jour de l'An"),
            ("2016-03-28", "Lundi de Pâques"),
            ("2016-05-01", "Fête du Travail"),
            ("2016-05-05", "Ascension"),
            ("2016-05-16", "Lundi de Pentecôte"),
            ("2016-06-10", "Fête de la Réconciliation"),
            ("2016-08-15", "Fête Nationale"),
            ("2016-11-01", "Toussaint"),
            ("2016-11-28", "Jour de la République"),
            ("2016-12-25", "Noël"),
        )

    def test_2017(self):
        # http://mokili.free.fr/jours_feries.php
        self.assertHolidaysInYear(
            2017,
            ("2017-01-01", "Jour de l'An"),
            ("2017-04-17", "Lundi de Pâques"),
            ("2017-05-01", "Fête du Travail"),
            ("2017-05-25", "Ascension"),
            ("2017-06-05", "Lundi de Pentecôte"),
            ("2017-06-10", "Fête de la Réconciliation"),
            ("2017-08-15", "Fête Nationale"),
            ("2017-11-01", "Toussaint"),
            ("2017-11-28", "Jour de la République"),
            ("2017-12-25", "Noël"),
        )

    def test_l10n_default(self):
        # http://mokili.free.fr/jours_feries.php
        self.assertLocalizedHolidays(
            ("2024-01-01", "Jour de l'An"),
            ("2024-04-01", "Lundi de Pâques"),
            ("2024-05-01", "Fête du Travail"),
            ("2024-05-09", "Ascension"),
            ("2024-05-20", "Lundi de Pentecôte"),
            ("2024-06-10", "Fête de la Réconciliation"),
            ("2024-08-15", "Fête Nationale"),
            ("2024-11-01", "Toussaint"),
            ("2024-11-28", "Jour de la République"),
            ("2024-12-25", "Noël"),
        )

    def test_l10n_en_us(self):
        # http://mokili.free.fr/jours_feries.php
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-09", "Ascension Day"),
            ("2024-05-20", "Whit Monday"),
            ("2024-06-10", "Reconciliation Day"),
            ("2024-08-15", "National Day"),
            ("2024-11-01", "All Saints' Day"),
            ("2024-11-28", "Republic Day"),
            ("2024-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, MAR
from holidays.calendars.julian import JULIAN_CALENDAR
from holidays.groups import (
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_SUN_TO_NEXT_WORKDAY


class Albania(
    ObservedHolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays, StaticHolidays
):
    """Albania holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Albania>
        * [Law No. 7651](https://web.archive.org/web/20250119183539/http://kqk.gov.al/sites/default/files/publikime/ligj_7651_-_per_festat_zyrtare_e_ditet_perkujtimore.pdf)
        * [Holidays for 2018-2024](https://web.archive.org/web/20250119183537/https://www.bankofalbania.org/Shtypi/Kalendari_i_festave_zyrtare_2024/)
    """

    country = "AL"
    default_language = "sq"
    # %s (estimated).
    estimated_label = tr("%s (e vlerësuar)")
    # %s (observed).
    observed_label = tr("%s (ditë pushimi e shtyrë)")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (ditë pushimi e shtyrë, e vlerësuar)")
    supported_languages = ("en_US", "sq", "uk")
    # Law No. 7651 from 21.12.1992.
    start_year = 1993

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=AlbaniaIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, AlbaniaStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_WORKDAY)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        dts_observed = set()

        # New Year's Day.
        name = tr("Festat e Vitit të Ri")
        dts_observed.add(self._add_new_years_day(name))
        dts_observed.add(self._add_new_years_day_two(name))

        if self._year >= 2004:
            # Summer Day.
            dts_observed.add(self._add_holiday_mar_14(tr("Dita e Verës")))

        if self._year >= 1996:
            # Nowruz Day.
            dts_observed.add(self._add_holiday_mar_22(tr("Dita e Nevruzit")))

        # Catholic Easter Sunday.
        dts_observed.add(self._add_easter_sunday(tr("E diela e Pashkëve Katolike")))

        dts_observed.add(
            # Orthodox Easter Sunday.
            self._add_easter_sunday(tr("E diela e Pashkëve Ortodokse"), JULIAN_CALENDAR)
        )

        # International Workers' Day.
        dts_observed.add(self._add_labor_day(tr("Dita Ndërkombëtare e Punëtorëve")))

        if 2004 <= self._year <= 2017:
            # Mother Teresa Beatification Day.
            dts_observed.add(self._add_holiday_oct_19(tr("Dita e Lumturimit të Shenjt Terezës")))
        elif self._year >= 2018:
            # Mother Teresa Canonization Day.
            dts_observed.add(self._add_holiday_sep_5(tr("Dita e Shenjtërimit të Shenjt Terezës")))

        if self._year >= 2024:
            # Alphabet Day.
            dts_observed.add(self._add_holiday_nov_22(tr("Dita e Alfabetit")))

        # Flag and Independence Day.
        dts_observed.add(self._add_holiday_nov_28(tr("Dita Flamurit dhe e Pavarësisë")))

        # Liberation Day.
        dts_observed.add(self._add_holiday_nov_29(tr("Dita e Çlirimit")))

        if self._year >= 2009:
            # National Youth Day.
            dts_observed.add(self._add_holiday_dec_8(tr("Dita Kombëtare e Rinisë")))

        # Christmas Day.
        dts_observed.add(self._add_christmas_day(tr("Krishtlindjet")))

        # Eid al-Fitr.
        dts_observed.update(self._add_eid_al_fitr_day(tr("Dita e Bajramit të Madh")))

        # Eid al-Adha.
        dts_observed.update(self._add_eid_al_adha_day(tr("Dita e Kurban Bajramit")))

        if self.observed:
            self._populate_observed(dts_observed)


class AL(Albania):
    pass


class ALB(Albania):
    pass


class AlbaniaIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2018, 2025)
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2018, 2025)


class AlbaniaStaticHolidays:
    # Public Holiday.
    public_holiday = tr("Ditë pushimi")

    special_public_holidays = {
        2020: (JAN, 3, public_holiday),
        2022: (MAR, 21, public_holiday),
        2024: (MAR, 15, public_holiday),
    }

    special_public_holidays_observed = {
        2007: (JAN, 3, tr("Dita e Kurban Bajramit")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import ARMENIAN, BANK, GOVERNMENT, HEBREW, ISLAMIC
from holidays.countries.argentina import Argentina
from tests.common import CommonCountryTests


class TestArgentina(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(
            Argentina,
            years_armenian=range(2007, 2050),
            years_bank=range(1975, 2050),
            years_government=range(2014, 2050),
            years_hebrew=range(1996, 2050),
            years_islamic=range(1997, 2050),
        )

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Argentina(categories=ARMENIAN, years=range(self.start_year, 2007)))
        self.assertNoHolidays(Argentina(categories=BANK, years=range(self.start_year, 1975)))
        self.assertNoHolidays(Argentina(categories=GOVERNMENT, years=range(self.start_year, 2014)))
        self.assertNoHolidays(Argentina(categories=HEBREW, years=range(self.start_year, 1996)))
        self.assertNoHolidays(Argentina(categories=ISLAMIC, years=range(self.start_year, 1997)))

    def test_special_holidays(self):
        self.assertHoliday(
            "2010-05-24",
            "2010-10-27",
            "2011-03-25",
            "2011-12-09",
            "2012-02-27",
            "2012-04-30",
            "2012-09-24",
            "2012-12-24",
            "2013-01-31",
            "2013-02-20",
            "2013-04-01",
            "2013-06-21",
            "2014-05-02",
            "2014-12-26",
            "2015-03-23",
            "2015-12-07",
            "2016-07-08",
            "2016-12-09",
            "2018-04-30",
            "2018-12-24",
            "2018-12-31",
            "2019-07-08",
            "2019-08-19",
            "2019-10-14",
            "2020-03-23",
            "2020-07-10",
            "2020-12-07",
            "2021-05-24",
            "2021-10-08",
            "2021-11-22",
            "2022-05-18",
            "2022-10-07",
            "2022-11-21",
            "2022-12-09",
            "2022-12-20",
            "2023-05-26",
            "2023-06-19",
            "2023-10-13",
            "2024-04-01",
            "2024-06-21",
            "2024-10-11",
            "2025-05-02",
            "2025-08-15",
            "2025-11-21",
            "2026-03-23",
            "2026-07-10",
            "2026-12-07",
        )

    def test_special_bank_holidays(self):
        self.assertBankHoliday(
            "2019-12-24",
            "2019-12-31",
            "2020-12-24",
            "2020-12-31",
            "2021-12-24",
            "2021-12-31",
            "2024-12-24",
            "2024-12-31",
            "2025-12-24",
            "2025-12-31",
        )

    def test_special_subdiv_holidays(self):
        # Buenos Aires.
        self.assertSubdivBHoliday("2018-11-30")

    def test_new_years_day(self):
        self.assertHolidayName("Año Nuevo", (f"{year}-01-01" for year in self.full_range))

    def test_epiphany_day(self):
        name = "Día de Reyes"
        self.assertHolidayName(name, (f"{year}-01-06" for year in range(self.start_year, 1977)))
        self.assertNoHolidayName(name, range(1977, self.end_year))

    def test_carnival_monday(self):
        name = "Lunes de Carnaval"
        self.assertHolidayName(
            name,
            "2011-03-07",
            "2016-02-08",
            "2017-02-27",
            "2018-02-12",
            "2019-03-04",
            "2020-02-24",
            "2021-02-15",
            "2022-02-28",
            "2023-02-20",
            "2024-02-12",
            "2025-03-03",
        )
        self.assertHolidayName(name, range(self.start_year, 1977), range(2011, self.end_year))
        self.assertNoHolidayName(name, range(1977, 2011))

    def test_carnival_tuesday(self):
        name = "Martes de Carnaval"
        self.assertHolidayName(
            name,
            "2011-03-08",
            "2016-02-09",
            "2017-02-28",
            "2018-02-13",
            "2019-03-05",
            "2020-02-25",
            "2021-02-16",
            "2022-03-01",
            "2023-02-21",
            "2024-02-13",
            "2025-03-04",
        )
        self.assertHolidayName(name, range(self.start_year, 1977), range(2011, self.end_year))
        self.assertNoHolidayName(name, range(1977, 2011))

    def test_national_day_of_remembrance(self):
        name = "Día Nacional de la Memoria por la Verdad y la Justicia"
        self.assertHolidayName(name, (f"{year}-03-24" for year in range(2006, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2006))

    def test_malvinas_war_day(self):
        name_1993 = "Día del Veterano de Guerra"
        name_2001 = "Día del Veterano y de los Caidos en la Guerra de Malvinas"
        self.assertHolidayName(
            name_1993,
            "1993-04-05",
            "1994-04-02",
            "1995-04-02",
            "1996-04-01",
            "1997-03-31",
            "1998-04-06",
            "1999-04-05",
            "2000-04-02",
        )
        self.assertHolidayName(
            name_2001,
            "2001-04-02",
            "2002-04-01",
            "2003-03-31",
            "2004-04-05",
            "2005-04-02",
            "2006-04-02",
            "2020-03-31",
        )
        self.assertHolidayName(
            name_2001,
            (f"{year}-04-02" for year in (*range(2007, 2020), *range(2021, self.end_year))),
        )
        self.assertNoHolidayName(
            name_1993, range(self.start_year, 1993), range(2001, self.end_year)
        )
        self.assertNoHolidayName(name_2001, range(self.start_year, 2001))

    def test_maundy_thursday(self):
        name = "Jueves Santo"
        self.assertHolidayName(
            name,
            "1976-04-15",
            "2011-04-21",
            "2012-04-05",
            "2013-03-28",
            "2014-04-17",
            "2015-04-02",
            "2016-03-24",
            "2017-04-13",
            "2018-03-29",
            "2019-04-18",
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertHolidayName(name, range(self.start_year, 1977), range(2011, self.end_year))
        self.assertNoHolidayName(name, range(1977, 2011))

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2016-03-25",
            "2017-04-14",
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(1977, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1977))

    def test_labor_day(self):
        self.assertHolidayName("Día del Trabajo", (f"{year}-05-01" for year in self.full_range))

    def test_may_revolution_day(self):
        self.assertHolidayName(
            "Día de la Revolución de Mayo", (f"{year}-05-25" for year in self.full_range)
        )

    def test_corpus_christi(self):
        name = "Corpus Christi"
        self.assertHolidayName(
            name,
            "1957-06-20",
            "1958-06-05",
            "1959-05-28",
            "1960-06-16",
            "1961-06-01",
            "1962-06-21",
            "1963-06-13",
            "1964-05-28",
            "1965-06-17",
            "1966-06-09",
            "1967-05-25",
            "1968-06-13",
            "1969-06-05",
            "1970-05-28",
            "1971-06-10",
            "1972-06-01",
            "1973-06-21",
            "1974-06-13",
            "1975-05-29",
        )
        self.assertNoHolidayName(name, range(1976, self.end_year))

    def test_sovereignty_over_malvinas_day(self):
        name = (
            "Día de la Afirmación de los Derechos Argentinos sobre las Malvinas, "
            "Islas y Sector Antártico"
        )
        self.assertHolidayName(
            name,
            "1983-04-02",
            "1984-06-10",
            "1985-06-10",
            "1986-06-10",
            "1987-06-10",
            "1988-06-13",
            "1989-06-10",
            "1990-06-10",
            "1991-06-10",
            "1992-06-08",
            "1993-06-14",
            "1994-06-13",
            "1995-06-10",
            "1996-06-10",
            "1997-06-09",
            "1998-06-08",
            "1999-06-14",
            "2000-06-10",
        )
        self.assertNoHolidayName(name, range(self.start_year, 1983), range(2001, self.end_year))

    def test_guemes_day(self):
        name = "Paso a la Inmortalidad del General Don Martín Miguel de Güemes"
        self.assertHolidayName(
            name,
            "2016-06-17",
            "2017-06-17",
            "2018-06-17",
            "2019-06-17",
            "2020-06-15",
            "2021-06-21",
            "2022-06-17",
            "2023-06-17",
            "2024-06-17",
            "2025-06-16",
        )
        self.assertHolidayName(name, range(2016, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2016))

    def test_belgrano_day(self):
        name = "Paso a la Inmortalidad del General Don Manuel Belgrano"
        self.assertHolidayName(
            name,
            (
                f"{year}-06-20"
                for year in (
                    *range(self.start_year, 1988),
                    *range(1992, 1995),
                    *range(2011, self.end_year),
                )
            ),
        )
        self.assertHolidayName(
            name,
            "1988-06-20",
            "1989-06-19",
            "1990-06-18",
            "1991-06-24",
            "1995-06-19",
            "1996-06-17",
            "1997-06-16",
            "1998-06-15",
            "1999-06-21",
            "2000-06-19",
            "2001-06-18",
            "2002-06-17",
            "2003-06-16",
            "2004-06-21",
            "2005-06-20",
            "2006-06-19",
            "2007-06-18",
            "2008-06-16",
            "2009-06-15",
            "2010-06-21",
        )

    def test_independence_day(self):
        self.assertHolidayName(
            "Día de la Independencia", (f"{year}-07-09" for year in self.full_range)
        )

    def test_san_martin_day(self):
        name = "Paso a la Inmortalidad del General Don José de San Martín"
        self.assertHolidayName(
            name,
            "2011-08-22",
            "2012-08-20",
            "2013-08-19",
            "2014-08-18",
            "2015-08-17",
            "2016-08-15",
            "2017-08-21",
            "2018-08-20",
            "2019-08-17",
            "2020-08-17",
            "2021-08-16",
            "2022-08-15",
            "2023-08-21",
            "2024-08-17",
            "2025-08-17",
        )
        self.assertHolidayName(name, self.full_range)

    def test_assumption_day(self):
        name = "Día de la Asunción"
        self.assertHolidayName(name, (f"{year}-08-15" for year in range(self.start_year, 1976)))
        self.assertNoHolidayName(name, range(1976, self.end_year))

    def test_cultural_diversity_day(self):
        name_1917 = "Día de la Raza"
        name_2010 = "Día del Respeto a la Diversidad Cultural"
        self.assertHolidayName(
            name_1917,
            (f"{year}-10-12" for year in (*range(self.start_year, 1976), *range(1982, 1988))),
        )
        self.assertHolidayName(
            name_1917,
            "1988-10-10",
            "1989-10-16",
            "1990-10-15",
            "1991-10-12",
            "1992-10-12",
            "1993-10-11",
            "1994-10-10",
            "1995-10-16",
            "1996-10-12",
            "1997-10-12",
            "1998-10-12",
            "1999-10-11",
            "2000-10-16",
            "2001-10-08",
            "2002-10-14",
            "2003-10-12",
            "2004-10-11",
            "2005-10-10",
            "2006-10-16",
            "2007-10-15",
            "2008-10-13",
            "2009-10-12",
        )
        self.assertHolidayName(
            name_2010,
            "2010-10-11",
            "2011-10-10",
            "2012-10-08",
            "2013-10-14",
            "2014-10-13",
            "2015-10-12",
            "2016-10-10",
            "2017-10-16",
            "2018-10-15",
            "2019-10-12",
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-16",
            "2024-10-12",
            "2025-10-12",
        )
        self.assertHolidayName(name_2010, range(2010, self.end_year))
        self.assertNoHolidayName(name_1917, range(1976, 1982), range(2010, self.end_year))
        self.assertNoHolidayName(name_2010, range(self.start_year, 2010))

    def test_all_saints_day(self):
        name = "Todos Los Santos"
        self.assertHolidayName(name, (f"{year}-11-01" for year in range(self.start_year, 1976)))
        self.assertNoHolidayName(name, range(1976, self.end_year))

    def test_national_sovereignty_day(self):
        name = "Día de la Soberanía Nacional"
        self.assertHolidayName(
            name,
            "2010-11-22",
            "2011-11-28",
            "2012-11-26",
            "2013-11-25",
            "2014-11-24",
            "2015-11-27",
            "2016-11-28",
            "2017-11-20",
            "2018-11-19",
            "2019-11-18",
            "2020-11-23",
            "2021-11-20",
            "2022-11-20",
            "2023-11-20",
            "2024-11-18",
            "2025-11-24",
        )
        self.assertHolidayName(name, range(2010, self.end_year))
        self.assertNoHolidayName(name, range(1976, 2010))

    def test_immaculate_conception_day(self):
        name = "Inmaculada Concepción de María"
        self.assertHolidayName(
            name,
            (
                f"{year}-12-08"
                for year in (*range(self.start_year, 1976), *range(1995, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(1976, 1995))

    def test_christmas_day(self):
        self.assertHolidayName("Navidad", (f"{year}-12-25" for year in self.full_range))

    def test_anniversary_of_battle_of_salta(self):
        name = "Aniversario de la Batalla de Salta"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "A":
                self.assertHolidayName(
                    name, holidays, (f"{year}-02-20" for year in range(1977, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1977))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_memory_of_guemes_salta(self):
        name = (
            "Dia de la memoria del Guerrero de la Independencia y Gobernador "
            "de la Provincia de Salta General Don Martín Miguel de Güemes"
        )
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "A":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-17" for year in range(1977, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1977))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_feasts_of_lord_and_virgin_of_miracle(self):
        name = "Festividades del Señor y de la Virgen del Milagro"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "A":
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-09-13" for year in range(1977, self.end_year)),
                    (f"{year}-09-14" for year in range(1977, self.end_year)),
                    (f"{year}-09-15" for year in range(1977, self.end_year)),
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1977))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_exaltation_of_holy_cross_day(self):
        name = "Día de la Exaltación de la Santa Cruz"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "D":
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-03" for year in range(2004, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2004))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_louis_day(self):
        name = "Día de San Luis Rey de Francia"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "D":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-25" for year in range(2004, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2004))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_day_of_remembrance_entre_rios(self):
        name = "Día Provincial de la Memoria por la Verdad y la Justicia"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "E":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-24" for year in range(2018, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2018))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_commemoration_of_battle_of_caseros(self):
        name = "Conmemoración de la Batalla de Caseros"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "E":
                self.assertHolidayName(
                    name, holidays, (f"{year}-02-03" for year in range(1984, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1984))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_state_workers_day_entre_rios(self):
        name = "Día del Trabajador Estatal"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "E":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-27" for year in range(2004, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2004))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_michael_archangels_day(self):
        name = "San Miguel Arcángel"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "E":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-29" for year in range(1993, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1993))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_death_of_juan_facundo_quiroga(self):
        name = "Día del fallecimiento de Juan Facundo Quiroga"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "F":
                self.assertHolidayName(
                    name, holidays, (f"{year}-02-16" for year in range(2021, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2021))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_provincial_autonomy_day(self):
        name = "Día de la Autonomía Provincial"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "F":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-01" for year in range(2020, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2020))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_day_of_remembrance_la_rioja(self):
        name = "Día de la Memoria por la Verdad y la Justicia"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "F":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-24" for year in range(2017, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2017))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_malvinas_memorial_day(self):
        name = "Día de los Caídos en Malvinas"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "F":
                self.assertHolidayName(
                    name, holidays, (f"{year}-04-02" for year in range(2017, 2021))
                )
                self.assertNoHolidayName(
                    name, holidays, range(self.start_year, 2017), range(2021, self.end_year)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_la_rioja_foundation_day(self):
        name = "Día de la fundación de La Rioja"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "F":
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-20" for year in range(2000, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2000))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_death_of_enrique_angelelli(self):
        name = "Día del Aniversario del Fallecimiento de Monseñor Enrique Angelelli"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "F":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-04" for year in range(2016, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2016))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_death_of_angel_vicente_penaloza(self):
        name = "Día del Aniversario del Fallecimiento de Ángel Vicente Peñaloza"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "F":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-12" for year in range(2020, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2020))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_tinkunaco_festival(self):
        name = "Día del Tinkunaco Riojano"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "F":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-31" for year in range(2000, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2000))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_teachers_day(self):
        name = "Día del Maestro"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "J":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-11" for year in range(2015, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2015))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_birthday_of_mamerto_esquiu(self):
        name = "Natalicio de Fray Mamerto Esquiú"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "K":
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-11" for year in range(1990, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1990))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_catamarca_autonomy_day(self):
        name = "Autonomía de Catamarca"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "K":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-25" for year in range(1990, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1990))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_miracle_day(self):
        name = "Día del Milagro"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "K":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-07" for year in range(2018, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2018))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_immaculate_conception_day_catamarca(self):
        name = "Inmaculada Concepción de María"
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "K":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-08" for year in range(1989, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(1976, 1989))
            else:
                self.assertNoHolidayName(name, holidays, range(1976, 1995))

    def test_saint_james_day(self):
        name = "Día del Apóstol Santiago"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "M":
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-25" for year in range(1977, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1977))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_anniversary_of_battle_of_tucuman(self):
        name = "Aniversario de la Batalla de Tucumán"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "T":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-24" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_plebiscite_1902_trevelin(self):
        name = "Plebiscito 1902 Trevelin"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "U":
                self.assertHolidayName(
                    name, holidays, (f"{year}-04-30" for year in range(1984, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1984))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_anniversary_of_arrival_of_first_welsh_settlers(self):
        name = "Aniversario del arribo de los primeros colonizadores galeses"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "U":
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-28" for year in range(1984, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1984))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_national_petroleum_day(self):
        name = "Día del Petróleo Nacional"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "U":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-13" for year in range(1984, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1984))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_tehuelches_and_mapuches_declare_loyalty(self):
        name = "Tehuelches y Mapuches declaran lealtad a la bandera Argentina"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "U":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-03" for year in range(2015, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2015))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_day_of_province_of_tierra_del_fuego(self):
        name = "Día de la Provincia de Tierra del Fuego, Antártida e Islas del Atlántico Sur"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "V":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-01" for year in range(1992, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1992))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_selknam_genocide_day(self):
        name = "Día del Genocidio Selk'Nam"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "V":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-25" for year in range(2021, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2021))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_anniversary_of_death_of_belgrano_corrientes(self):
        name = (
            "Día del Aniversario del Fallecimiento del General Manuel José Joaquín "
            "del Corazón de Jesús Belgrano"
        )
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "W":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-20" for year in range(2009, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2009))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_anniversary_of_death_of_san_martin_corrientes(self):
        name = "Día del Aniversario del Fallecimiento del General José Francisco de San Martín"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "W":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-17" for year in range(2009, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2009))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_carnival_monday_jujuy(self):
        name = "Lunes de Carnaval"
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "Y":
                self.assertHolidayName(
                    name,
                    holidays,
                    "1984-03-05",
                    "1985-02-18",
                    "1986-02-10",
                    "1987-03-02",
                    "1990-02-26",
                    "1991-02-11",
                    "1992-03-02",
                    "1993-02-22",
                    "1994-02-14",
                    "2005-02-07",
                    "2006-02-27",
                    "2007-02-19",
                    "2008-02-04",
                    "2009-02-23",
                    "2010-02-15",
                )
                self.assertHolidayName(name, holidays, range(1984, self.end_year))
                self.assertNoHolidayName(name, holidays, range(1977, 1984))
            else:
                self.assertNoHolidayName(name, holidays, range(1977, 2011))

    def test_carnival_tuesday_jujuy(self):
        name = "Martes de Carnaval"
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "Y":
                self.assertHolidayName(
                    name,
                    holidays,
                    "1984-03-06",
                    "1985-02-19",
                    "1986-02-11",
                    "1987-03-03",
                    "1990-02-27",
                    "1991-02-12",
                    "1992-03-03",
                    "1993-02-23",
                    "1994-02-15",
                    "2005-02-08",
                    "2006-02-28",
                    "2007-02-20",
                    "2008-02-05",
                    "2009-02-24",
                    "2010-02-16",
                )
                self.assertHolidayName(name, holidays, range(1984, self.end_year))
                self.assertNoHolidayName(name, holidays, range(1977, 1984))
            else:
                self.assertNoHolidayName(name, holidays, range(1977, 2011))

    def test_jujuy_exodus_day(self):
        name = "Día del Éxodo Jujeño"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "Y":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-23" for year in range(1984, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1984))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_jujuy_political_autonomy_day(self):
        name = "Autonomía Política de Jujuy"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "Y":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-18" for year in range(1984, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1984))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_pachamama_day(self):
        name = "Día de la Pachamama"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "Y":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-01" for year in range(1996, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1996))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_day_of_virgin_of_rosary_of_rio_blanco_and_paypaya(self):
        name = "Día de la Virgen del Rosario de Río Blanco y Paypaya"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "Y":
                self.assertHolidayName(
                    name, holidays, (f"{year}-10-07" for year in range(1997, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1997))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_great_day_of_jujuy(self):
        name = "Día Grande de Jujuy"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "Y":
                self.assertHolidayName(
                    name, holidays, (f"{year}-04-27" for year in range(2021, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2021))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_john_boscos_day(self):
        name = "Homenaje al Patrono de la Provincia San Juan Bosco"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "Z":
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-01-31" for year in range(2007, 2015)),
                    (f"{year}-08-16" for year in range(2015, self.end_year)),
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2007))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_anniversary_of_death_of_nestor_carlos_kirchner(self):
        name = (
            "Día del Aniversario del Fallecimiento del ex Presidente de la Nación "
            "Doctor Néstor Carlos Kirchner"
        )
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "Z":
                self.assertHolidayName(
                    name, holidays, (f"{year}-10-27" for year in range(2014, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2014))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_commemoration_of_workers_shot_in_patagonian_strikes(self):
        name = "Conmemoración a los obreros fusilados en las Huelgas Patagónicas"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "Z":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-07" for year in range(2019, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 2019))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_day_of_action_for_tolerance_and_respect_among_peoples(self):
        name = "Día de acción por la tolerancia y el respeto entre los pueblos"
        self.assertNoHolidayName(name)
        self.assertArmenianHolidayName(
            name, (f"{year}-04-24" for year in range(2007, self.end_year))
        )
        self.assertNoArmenianHolidayName(name, range(self.start_year, 2007))

    def test_bankers_day(self):
        name = "Día del Bancario"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-11-06" for year in range(1975, self.end_year)))
        self.assertNoBankHolidayName(name, range(self.start_year, 1975))

    def test_state_workers_day(self):
        name = "Día del Trabajador del Estado"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(
            name, (f"{year}-06-27" for year in range(2014, self.end_year))
        )
        self.assertNoGovernmentHolidayName(name, range(self.start_year, 2014))

    def test_rosh_hashanah(self):
        name = "Año Nuevo Judío (Rosh Hashana)"
        self.assertNoHolidayName(name)
        self.assertHebrewHolidayName(
            name,
            "1996-09-14",
            "1996-09-15",
            "1997-10-02",
            "1997-10-03",
            "2018-09-10",
            "2018-09-11",
            "2019-09-30",
            "2019-10-01",
            "2020-09-19",
            "2020-09-20",
            "2021-09-07",
            "2021-09-08",
            "2022-09-26",
            "2022-09-27",
            "2023-09-16",
            "2023-09-17",
            "2024-10-03",
            "2024-10-04",
            "2025-09-23",
            "2025-09-24",
        )
        self.assertHebrewHolidayNameCount(name, 2, range(1996, self.end_year))
        self.assertNoHebrewHolidayName(name, range(self.start_year, 1996))

    def test_yom_kippur(self):
        name = "Día del Perdón (Iom Kipur)"
        self.assertNoHolidayName(name)
        self.assertHebrewHolidayName(
            name,
            "1996-09-23",
            "1997-10-11",
            "2018-09-19",
            "2019-10-09",
            "2020-09-28",
            "2021-09-16",
            "2022-10-05",
            "2023-09-25",
            "2024-10-12",
            "2025-10-02",
        )
        self.assertHebrewHolidayName(name, range(1996, self.end_year))
        self.assertNoHebrewHolidayName(name, range(self.start_year, 1996))

    def test_pesach(self):
        name = "Pascua Judía (Pésaj)"
        self.assertNoHolidayName(name)
        self.assertHebrewHolidayName(
            name,
            "2007-04-03",
            "2007-04-04",
            "2007-04-09",
            "2007-04-10",
            "2008-04-20",
            "2008-04-21",
            "2008-04-26",
            "2008-04-27",
            "2018-03-31",
            "2018-04-01",
            "2018-04-06",
            "2018-04-07",
            "2019-04-20",
            "2019-04-21",
            "2019-04-26",
            "2019-04-27",
            "2020-04-09",
            "2020-04-10",
            "2020-04-15",
            "2020-04-16",
            "2021-03-28",
            "2021-03-29",
            "2021-04-03",
            "2021-04-04",
            "2022-04-16",
            "2022-04-17",
            "2022-04-22",
            "2022-04-23",
            "2023-04-06",
            "2023-04-07",
            "2023-04-12",
            "2023-04-13",
            "2024-04-23",
            "2024-04-24",
            "2024-04-29",
            "2024-04-30",
            "2025-04-13",
            "2025-04-14",
            "2025-04-19",
            "2025-04-20",
        )
        self.assertHebrewHolidayNameCount(name, 4, range(2007, self.end_year))
        self.assertNoHebrewHolidayName(name, range(self.start_year, 2007))

    def test_islamic_new_year(self):
        name = "Año Nuevo Musulmán (Hégira)"
        self.assertNoHolidayName(name)
        self.assertIslamicIslamicNoEstimatedHolidayName(
            name,
            "2019-08-31",
            "2020-08-20",
            "2021-08-08",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertIslamicIslamicNoEstimatedHolidayName(name, range(1997, self.end_year))
        self.assertNoIslamicIslamicNoEstimatedHolidayName(name, range(self.start_year, 1997))

    def test_eid_al_fitr(self):
        name = "Día posterior a la culminación del ayuno (Id Al-Fitr)"
        self.assertNoHolidayName(name)
        self.assertIslamicIslamicNoEstimatedHolidayName(
            name,
            "2019-06-04",
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-31",
        )
        self.assertIslamicIslamicNoEstimatedHolidayName(name, range(1997, self.end_year))
        self.assertNoIslamicIslamicNoEstimatedHolidayName(name, range(self.start_year, 1997))

    def test_eid_al_adha(self):
        name = "Día de la Fiesta del Sacrificio (Id Al-Adha)"
        self.assertNoHolidayName(name)
        self.assertIslamicIslamicNoEstimatedHolidayName(
            name,
            "2019-08-11",
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-10",
        )
        self.assertIslamicIslamicNoEstimatedHolidayName(name, range(1997, self.end_year))
        self.assertNoIslamicIslamicNoEstimatedHolidayName(name, range(self.start_year, 1997))

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Año Nuevo"),
            ("2022-02-28", "Lunes de Carnaval"),
            ("2022-03-01", "Martes de Carnaval"),
            ("2022-03-24", "Día Nacional de la Memoria por la Verdad y la Justicia"),
            ("2022-04-02", "Día del Veterano y de los Caidos en la Guerra de Malvinas"),
            ("2022-04-14", "Jueves Santo"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-05-01", "Día del Trabajo"),
            ("2022-05-18", "Censo Nacional 2022"),
            ("2022-05-25", "Día de la Revolución de Mayo"),
            ("2022-06-17", "Paso a la Inmortalidad del General Don Martín Miguel de Güemes"),
            ("2022-06-20", "Paso a la Inmortalidad del General Don Manuel Belgrano"),
            ("2022-07-09", "Día de la Independencia"),
            ("2022-08-15", "Paso a la Inmortalidad del General Don José de San Martín"),
            ("2022-10-07", "Feriado con fines turísticos"),
            ("2022-10-10", "Día del Respeto a la Diversidad Cultural"),
            ("2022-11-20", "Día de la Soberanía Nacional"),
            ("2022-11-21", "Feriado con fines turísticos"),
            ("2022-12-08", "Inmaculada Concepción de María"),
            ("2022-12-09", "Feriado con fines turísticos"),
            ("2022-12-20", "Día de la Victoria de la Copa Mundial de la FIFA 2022"),
            ("2022-12-25", "Navidad"),
        )

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "Año Nuevo"),
            ("2023-02-20", "Lunes de Carnaval"),
            ("2023-02-21", "Martes de Carnaval"),
            ("2023-03-24", "Día Nacional de la Memoria por la Verdad y la Justicia"),
            ("2023-04-02", "Día del Veterano y de los Caidos en la Guerra de Malvinas"),
            ("2023-04-06", "Jueves Santo"),
            ("2023-04-07", "Viernes Santo"),
            ("2023-05-01", "Día del Trabajo"),
            ("2023-05-25", "Día de la Revolución de Mayo"),
            ("2023-05-26", "Feriado con fines turísticos"),
            ("2023-06-17", "Paso a la Inmortalidad del General Don Martín Miguel de Güemes"),
            ("2023-06-19", "Feriado con fines turísticos"),
            ("2023-06-20", "Paso a la Inmortalidad del General Don Manuel Belgrano"),
            ("2023-07-09", "Día de la Independencia"),
            ("2023-08-21", "Paso a la Inmortalidad del General Don José de San Martín"),
            ("2023-10-13", "Feriado con fines turísticos"),
            ("2023-10-16", "Día del Respeto a la Diversidad Cultural"),
            ("2023-11-20", "Día de la Soberanía Nacional"),
            ("2023-12-08", "Inmaculada Concepción de María"),
            ("2023-12-25", "Navidad"),
        )

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "Año Nuevo"),
            ("2024-02-12", "Lunes de Carnaval"),
            ("2024-02-13", "Martes de Carnaval"),
            ("2024-03-24", "Día Nacional de la Memoria por la Verdad y la Justicia"),
            ("2024-03-28", "Jueves Santo"),
            ("2024-03-29", "Viernes Santo"),
            ("2024-04-01", "Feriado con fines turísticos"),
            ("2024-04-02", "Día del Veterano y de los Caidos en la Guerra de Malvinas"),
            ("2024-05-01", "Día del Trabajo"),
            ("2024-05-25", "Día de la Revolución de Mayo"),
            ("2024-06-17", "Paso a la Inmortalidad del General Don Martín Miguel de Güemes"),
            ("2024-06-20", "Paso a la Inmortalidad del General Don Manuel Belgrano"),
            ("2024-06-21", "Feriado con fines turísticos"),
            ("2024-07-09", "Día de la Independencia"),
            ("2024-08-17", "Paso a la Inmortalidad del General Don José de San Martín"),
            ("2024-10-11", "Feriado con fines turísticos"),
            ("2024-10-12", "Día del Respeto a la Diversidad Cultural"),
            ("2024-11-18", "Día de la Soberanía Nacional"),
            ("2024-12-08", "Inmaculada Concepción de María"),
            ("2024-12-25", "Navidad"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Año Nuevo"),
            ("2022-02-03", "Conmemoración de la Batalla de Caseros"),
            ("2022-02-16", "Día del fallecimiento de Juan Facundo Quiroga"),
            ("2022-02-20", "Aniversario de la Batalla de Salta"),
            ("2022-02-28", "Lunes de Carnaval"),
            ("2022-03-01", "Día de la Autonomía Provincial; Martes de Carnaval"),
            (
                "2022-03-24",
                "Día Nacional de la Memoria por la Verdad y la Justicia; "
                "Día Provincial de la Memoria por la Verdad y la Justicia; "
                "Día de la Memoria por la Verdad y la Justicia",
            ),
            ("2022-04-02", "Día del Veterano y de los Caidos en la Guerra de Malvinas"),
            ("2022-04-14", "Jueves Santo"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-04-16", "Pascua Judía (Pésaj)"),
            ("2022-04-17", "Pascua Judía (Pésaj)"),
            ("2022-04-22", "Pascua Judía (Pésaj)"),
            ("2022-04-23", "Pascua Judía (Pésaj)"),
            ("2022-04-24", "Día de acción por la tolerancia y el respeto entre los pueblos"),
            ("2022-04-27", "Día Grande de Jujuy"),
            ("2022-04-30", "Plebiscito 1902 Trevelin"),
            ("2022-05-01", "Día del Trabajo"),
            ("2022-05-02", "Día posterior a la culminación del ayuno (Id Al-Fitr)"),
            ("2022-05-03", "Día de la Exaltación de la Santa Cruz"),
            ("2022-05-11", "Natalicio de Fray Mamerto Esquiú"),
            ("2022-05-18", "Censo Nacional 2022"),
            ("2022-05-20", "Día de la fundación de La Rioja"),
            ("2022-05-25", "Día de la Revolución de Mayo"),
            (
                "2022-06-01",
                "Día de la Provincia de Tierra del Fuego, Antártida e Islas del Atlántico Sur",
            ),
            (
                "2022-06-17",
                "Dia de la memoria del Guerrero de la Independencia y Gobernador de la Provincia "
                "de Salta General Don Martín Miguel de Güemes; "
                "Paso a la Inmortalidad del General Don Martín Miguel de Güemes",
            ),
            (
                "2022-06-20",
                "Día del Aniversario del Fallecimiento del General Manuel José Joaquín "
                "del Corazón de Jesús Belgrano; "
                "Paso a la Inmortalidad del General Don Manuel Belgrano",
            ),
            ("2022-06-27", "Día del Trabajador Estatal; Día del Trabajador del Estado"),
            (
                "2022-07-09",
                "Día de la Fiesta del Sacrificio (Id Al-Adha); Día de la Independencia",
            ),
            ("2022-07-25", "Día del Apóstol Santiago"),
            ("2022-07-28", "Aniversario del arribo de los primeros colonizadores galeses"),
            ("2022-07-30", "Año Nuevo Musulmán (Hégira)"),
            ("2022-08-01", "Día de la Pachamama"),
            ("2022-08-04", "Día del Aniversario del Fallecimiento de Monseñor Enrique Angelelli"),
            ("2022-08-15", "Paso a la Inmortalidad del General Don José de San Martín"),
            ("2022-08-16", "Homenaje al Patrono de la Provincia San Juan Bosco"),
            (
                "2022-08-17",
                "Día del Aniversario del Fallecimiento del General José Francisco de San Martín",
            ),
            ("2022-08-23", "Día del Éxodo Jujeño"),
            ("2022-08-25", "Autonomía de Catamarca; Día de San Luis Rey de Francia"),
            ("2022-09-07", "Día del Milagro"),
            ("2022-09-11", "Día del Maestro"),
            ("2022-09-13", "Festividades del Señor y de la Virgen del Milagro"),
            ("2022-09-14", "Festividades del Señor y de la Virgen del Milagro"),
            ("2022-09-15", "Festividades del Señor y de la Virgen del Milagro"),
            ("2022-09-24", "Aniversario de la Batalla de Tucumán"),
            ("2022-09-26", "Año Nuevo Judío (Rosh Hashana)"),
            ("2022-09-27", "Año Nuevo Judío (Rosh Hashana)"),
            ("2022-09-29", "San Miguel Arcángel"),
            ("2022-10-05", "Día del Perdón (Iom Kipur)"),
            (
                "2022-10-07",
                "Día de la Virgen del Rosario de Río Blanco y Paypaya; "
                "Feriado con fines turísticos",
            ),
            ("2022-10-10", "Día del Respeto a la Diversidad Cultural"),
            (
                "2022-10-27",
                "Día del Aniversario del Fallecimiento del ex Presidente de la Nación "
                "Doctor Néstor Carlos Kirchner",
            ),
            ("2022-11-03", "Tehuelches y Mapuches declaran lealtad a la bandera Argentina"),
            ("2022-11-06", "Día del Bancario"),
            ("2022-11-12", "Día del Aniversario del Fallecimiento de Ángel Vicente Peñaloza"),
            ("2022-11-18", "Autonomía Política de Jujuy"),
            ("2022-11-20", "Día de la Soberanía Nacional"),
            ("2022-11-21", "Feriado con fines turísticos"),
            ("2022-11-25", "Día del Genocidio Selk'Nam"),
            ("2022-12-07", "Conmemoración a los obreros fusilados en las Huelgas Patagónicas"),
            ("2022-12-08", "Inmaculada Concepción de María"),
            ("2022-12-09", "Feriado con fines turísticos"),
            ("2022-12-13", "Día del Petróleo Nacional"),
            ("2022-12-20", "Día de la Victoria de la Copa Mundial de la FIFA 2022"),
            ("2022-12-25", "Navidad"),
            ("2022-12-31", "Día del Tinkunaco Riojano"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-02-03", "Commemoration of the Battle of Caseros"),
            ("2022-02-16", "Day of the Death of Juan Facundo Quiroga"),
            ("2022-02-20", "Anniversary of the Battle of Salta"),
            ("2022-02-28", "Carnival Monday"),
            ("2022-03-01", "Carnival Tuesday; Provincial Autonomy Day"),
            (
                "2022-03-24",
                "Day of Remembrance for Truth and Justice; "
                "National Day of Remembrance for Truth and Justice; "
                "Provincial Day of Remembrance for Truth and Justice",
            ),
            ("2022-04-02", "Veteran's Day and the Fallen in the Malvinas War"),
            ("2022-04-14", "Maundy Thursday"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-16", "Pesach"),
            ("2022-04-17", "Pesach"),
            ("2022-04-22", "Pesach"),
            ("2022-04-23", "Pesach"),
            ("2022-04-24", "Day of Action for Tolerance and Respect among Peoples"),
            ("2022-04-27", "Great Day of Jujuy"),
            ("2022-04-30", "Plebiscite 1902 Trevelin"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-02", "Eid al-Fitr"),
            ("2022-05-03", "Exaltation of the Holy Cross Day"),
            ("2022-05-11", "Birthday of Mamerto Esquiú"),
            ("2022-05-18", "National Census Day 2022"),
            ("2022-05-20", "La Rioja Foundation Day"),
            ("2022-05-25", "May Revolution Day"),
            (
                "2022-06-01",
                "Day of the Province of Tierra del Fuego, Antarctica and the South "
                "Atlantic Islands",
            ),
            (
                "2022-06-17",
                "Day of Memory of General Don Martín Miguel de Güemes; "
                "Pass to the Immortality of General Don Martín Miguel de Güemes",
            ),
            (
                "2022-06-20",
                "Anniversary of the Death of General Manuel Belgrano; "
                "Pass to the Immortality of General Don Manuel Belgrano",
            ),
            ("2022-06-27", "State Worker's Day"),
            ("2022-07-09", "Eid al-Adha; Independence Day"),
            ("2022-07-25", "Saint James' Day"),
            ("2022-07-28", "Anniversary of the arrival of the first Welsh settlers"),
            ("2022-07-30", "Islamic New Year"),
            ("2022-08-01", "Pachamama Day"),
            ("2022-08-04", "Anniversary of the Death of Enrique Angelelli"),
            ("2022-08-15", "Pass to the Immortality of General Don José de San Martín"),
            ("2022-08-16", "Saint John Bosco's Day"),
            ("2022-08-17", "Anniversary of the Death of General José Francisco de San Martín"),
            ("2022-08-23", "Jujuy Exodus Day"),
            ("2022-08-25", "Catamarca Autonomy Day; Saint Louis the King of France's Day"),
            ("2022-09-07", "Miracle Day"),
            ("2022-09-11", "Teacher's Day"),
            ("2022-09-13", "Feasts of the Lord and the Virgin of Miracle"),
            ("2022-09-14", "Feasts of the Lord and the Virgin of Miracle"),
            ("2022-09-15", "Feasts of the Lord and the Virgin of Miracle"),
            ("2022-09-24", "Anniversary of the Battle of Tucumán"),
            ("2022-09-26", "Rosh Hashanah"),
            ("2022-09-27", "Rosh Hashanah"),
            ("2022-09-29", "Saint Michael the Archangel's Day"),
            ("2022-10-05", "Yom Kippur"),
            (
                "2022-10-07",
                "Bridge Public Holiday; Day of the Virgin of the Rosary of Río Blanco and Paypaya",
            ),
            ("2022-10-10", "Respect for Cultural Diversity Day"),
            ("2022-10-27", "Anniversary of the Death of Néstor Carlos Kirchner"),
            ("2022-11-03", "Tehuelches and Mapuches declare loyalty to the Argentine flag"),
            ("2022-11-06", "Bankers' Day"),
            ("2022-11-12", "Anniversary of the Death of Ángel Vicente Peñaloza"),
            ("2022-11-18", "Jujuy Political Autonomy Day"),
            ("2022-11-20", "National Sovereignty Day"),
            ("2022-11-21", "Bridge Public Holiday"),
            ("2022-11-25", "Selk'Nam Genocide Day"),
            ("2022-12-07", "Commemoration of the workers shot in the Patagonian Strikes"),
            ("2022-12-08", "Immaculate Conception"),
            ("2022-12-09", "Bridge Public Holiday"),
            ("2022-12-13", "National Petroleum Day"),
            ("2022-12-20", "FIFA World Cup 2022 Victory Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-31", "Tinkunaco Festival"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-02-03", "День битви під Касеросом"),
            ("2022-02-16", "День смерті Хуана Факундо Кіроги"),
            ("2022-02-20", "Річниця битви при Сальті"),
            ("2022-02-28", "Карнавальний понеділок"),
            ("2022-03-01", "День провінційної автономії; Карнавальний вівторок"),
            (
                "2022-03-24",
                "День памʼяті заради правди та правосуддя; "
                "Національний день памʼяті заради правди та правосуддя; "
                "Провінційний день памʼяті заради правди та правосуддя",
            ),
            ("2022-04-02", "День ветеранів та загиблих на Мальвінській війні"),
            ("2022-04-14", "Великий четвер"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-16", "Песах"),
            ("2022-04-17", "Песах"),
            ("2022-04-22", "Песах"),
            ("2022-04-23", "Песах"),
            ("2022-04-24", "День дій на підтримку толерантності та поваги між народами"),
            ("2022-04-27", "Великий День Хухуя"),
            ("2022-04-30", "Річниця плебісциту 1902 року"),
            ("2022-05-01", "День праці"),
            ("2022-05-02", "Рамазан-байрам"),
            ("2022-05-03", "День Воздвиження Хреста Господнього"),
            ("2022-05-11", "День народження Мамерто Ескуї"),
            ("2022-05-18", "День національного перепису 2022"),
            ("2022-05-20", "День заснування Ла-Ріохи"),
            ("2022-05-25", "День Травневої революції"),
            (
                "2022-06-01",
                "День провінції Вогняна Земля, Антарктиди і Південноатлантичних островів",
            ),
            ("2022-06-17", "День памʼяті генерала Мартіна Мігеля де Гуемеса"),
            (
                "2022-06-20",
                "День памʼяті генерала Мануеля Бельграно; День смерті генерала Мануеля Бельграно",
            ),
            ("2022-06-27", "День державного службовця"),
            ("2022-07-09", "День незалежності; Курбан-байрам"),
            ("2022-07-25", "День Святого Якова"),
            ("2022-07-28", "Річниця прибуття перших валлійських поселенців"),
            ("2022-07-30", "Ісламський Новий рік"),
            ("2022-08-01", "День Пачамами"),
            ("2022-08-04", "День смерті Енріке Анхелельї"),
            ("2022-08-15", "День памʼяті генерала Хосе де Сан-Мартіна"),
            ("2022-08-16", "День Святого Івана Боско"),
            ("2022-08-17", "День смерті генерала Хосе де Сан-Мартіна"),
            ("2022-08-23", "День Виходу Хухуя"),
            ("2022-08-25", "День Святого Людовика; День автономії Катамарки"),
            ("2022-09-07", "День дива"),
            ("2022-09-11", "День учителя"),
            ("2022-09-13", "Свято Господа та Богородиці Чуда"),
            ("2022-09-14", "Свято Господа та Богородиці Чуда"),
            ("2022-09-15", "Свято Господа та Богородиці Чуда"),
            ("2022-09-24", "Річниця битви при Тукумані"),
            ("2022-09-26", "Рош га-Шана"),
            ("2022-09-27", "Рош га-Шана"),
            ("2022-09-29", "День Святого Архангела Михаїла"),
            ("2022-10-05", "Йом Кіпур"),
            ("2022-10-07", "День Богородиці Вервиці Ріо-Бланко і Пайпаї; Додатковий вихідний"),
            ("2022-10-10", "День поваги до культурного різноманіття"),
            ("2022-10-27", "День смерті Нестора Карлоса Кіршнера"),
            ("2022-11-03", "День присяги теуелче та мапуче на вірність аргентинському прапору"),
            ("2022-11-06", "День банківських працівників"),
            ("2022-11-12", "День смерті Анхеля Вісенте Пеньялоса"),
            ("2022-11-18", "День політичної автономії Хухуя"),
            ("2022-11-20", "День національного суверенітету"),
            ("2022-11-21", "Додатковий вихідний"),
            ("2022-11-25", "День геноциду народу селькнам"),
            (
                "2022-12-07",
                "Вшанування пам'яті робітників, розстріляних під час Патагонських страйків",
            ),
            ("2022-12-08", "Непорочне зачаття Діви Марії"),
            ("2022-12-09", "Додатковий вихідний"),
            ("2022-12-13", "Національний день нафти"),
            ("2022-12-20", "День перемоги збірної Аргентини на Чемпіонаті світу з футболу 2022"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-31", "Свято Тінкунако"),
        )
